
#ifndef __BASEDEFS_H__
#define __BASEDEFS_H__


	#include <stdlib.h>
	#include <string.h>

	#ifdef __cplusplus
		#define BEGIN_EXTERNC() extern "C" \
			{
		#define END_EXTERNC() };
	#else
		#define BEGIN_EXTERNC()
		#define END_EXTERNC()
	#endif
	

	#if !defined(__cplusplus) || defined(DIRECTENGINE_COMPILE)
		#define COMPILE_WITH_C
	#endif	
	
	#ifdef COMPILE_WITH_C
		#define LPBASECLASS struct BaseClass_t * 
		#define	LPAGGREGATE struct Aggregate_t *
	#else
		class BaseClass;
		#define LPBASECLASS BaseClass*
		class Aggregate;
		#define LPAGGREGATE Aggregate*
	#endif

	
	typedef float DFLOAT;
	

	#define DEFINE_HANDLE_TYPE(name) \
		typedef struct {int __nothing;} *name;


	#define MAX_CS_FILENAME_LEN	100


	// Model state flags.
	#define MS_PLAYDONE		1	// The (nonlooping) model has finished playing the animation.


	// Handle definitions.. it defines them in this wierd way so 
	// you can't accidentally cast between them.
	DEFINE_HANDLE_TYPE(HCLASS)
	DEFINE_HANDLE_TYPE(HSURFACE)
	DEFINE_HANDLE_TYPE(HDEFONT)
	DEFINE_HANDLE_TYPE(HMESSAGEREAD)
	DEFINE_HANDLE_TYPE(HMESSAGEWRITE)
	DEFINE_HANDLE_TYPE(HSTRING)
	DEFINE_HANDLE_TYPE(HDECOLOR)
	DEFINE_HANDLE_TYPE(HSOUNDDE)
	DEFINE_HANDLE_TYPE(HCLIENT);
	DEFINE_HANDLE_TYPE(HATTACHMENT);

	typedef unsigned long HMODELNODE;
	#define INVALID_MODEL_NODE ((HMODELNODE)-1)

	DEFINE_HANDLE_TYPE(HLOCALOBJ)	// Client-side object handle.
	typedef HLOCALOBJ HOBJECT;		// Server-side object handle.

	// ---------------------------------------------------------------- //
	// Here is the layout of all message IDs.
	// ---------------------------------------------------------------- //
	
	// Server->Client (0-255):
	// DirectEngine reserves 0-15 and defines:
	#define STC_BPRINT	0	// Used for debugging.. print a message in the console.

	// Client->Server (0-255):
	// DirectEngine reserves 0-15.

	// Object->Object (0-65k)
	// 0-1000 reserved.
	// 1000-2000 shared general messages (damage, I see you, etc..)
	// 2000-2500 Riot
	// 3500-4000 Blood 2
	// 4000-4500 Dreadon


	// This is used all over to classify which side of a polygon something's on.
	typedef enum
	{
		BackSide=0,
		FrontSide=1,
		Intersect=2
	} PolySide;


	
	// Include really base stuff.
	#include "basetypes_de.h"
	#include "de_codes.h"


	// This structure is used when creating objects.  When you want to
	// create an object, you call ServerDE::CreateObject with one of these.
	// The structure you pass in is passed to the object's PostPropRead function,
	// where it can override whatever it wants to.
	typedef struct ObjectCreateStruct_t
	{
		// Main info.
		unsigned short	m_ObjectType;
		DDWORD			m_Flags;
		DVector			m_Pos;
		DVector			m_Scale;
		DRotation		m_Rotation;
		D_WORD			m_ContainerCode;	// Container code if it's a container.  It's in here because
											// you can only set it at creation time.

		DDWORD			m_UserData;			// User data


		char			m_Filename[MAX_CS_FILENAME_LEN+1];	// This is the model, sound, or sprite filename.
															// It also can be the WorldModel name.
															// This can be zero-length when it's not needed.
		
		char			m_SkinName[MAX_CS_FILENAME_LEN+1];	// This can be zero-length.. if you set it for an 
															// OT_MODEL, it's the skin filename.	  

		// Server only info.
		char			m_Name[MAX_CS_FILENAME_LEN+1];		// This object's name.
		float			m_NextUpdate;						// This will be the object's starting NextUpdate.
		float			m_fDeactivationTime;				// Amount of time before object deactivates self
	} ObjectCreateStruct;

	// Initialize an ObjectCreateStruct.
	#define INIT_OBJECTCREATESTRUCT(theStruct) \
	{ \
		memset(&theStruct, 0, sizeof(theStruct));\
		VEC_SET(theStruct.m_Scale, 1, 1, 1); \
		theStruct.m_Rotation.m_Spin = 1.0f; \
	}


	// Model hook.
	#define MHF_USETEXTURE		1	// Use the texture for this model (if any).
	typedef struct ModelHookData_t
	{
		HLOCALOBJ		m_hObject;
		unsigned long	m_Flags;		// Combination of MHF_ flags above.
		unsigned long	m_ObjectFlags;	// The model's object flags.  You can modify them
										// in here without changing the model's flags
										// permanently.
		DVector			*m_LightAdd;	// RGB 0-255
	} ModelHookData;


	// Render modes are what are used to describe a video mode/video card.
	typedef struct RMode_t
	{
		DBOOL			m_bHardware;

		char			m_RenderDLL[200];		// What DLL this comes from.		
		char			m_InternalName[100];	// This is what the DLLs use to identify a card.
		char			m_Description[100];		// This is a 'friendly' string describing the card.
		
		DDWORD			m_Width, m_Height, m_BitDepth;
		struct RMode_t	*m_pNext;
	} RMode;



#endif  // __BASEDEFS_H__


