
#ifndef __BASETYPES_DE_H__
#define __BASETYPES_DE_H__


	#include <math.h>

	#include "de_codes.h"


	#define INLINE_FN __inline


	// Base types.
	#define DNULL 0
	#define DBOOL char
	#define DTRUE 1
	#define DFALSE 0
	#define DDWORD unsigned long
	#define D_WORD unsigned short
	#define DBYTE unsigned char


	// Reference to a surface.
	typedef unsigned long HPOLY;
	#define INVALID_HPOLY 0xFFFFFFFF


	// Maximum number of sky objects.
	#define MAX_SKYOBJECTS	30


	// Parsing stuff.
	#define PARSE_MAXTOKENS		64
	#define PARSE_MAXTOKENSIZE	80

	// Helpful math definitions.
	#define MATH_PI				3.141592653589793f
	#define MATH_HALFPI			1.570796326795f
	#define MATH_CIRCLE			6.283185307178f
	#define MATH_ONE_OVER_PI	0.3183098861839f
	#define MATH_EPSILON		0.00000000001f
	#define MATH_DEGREES_TO_RADIANS(x) ((x) *  0.01745329251994f)

	#define MATH_ONE_OVER_255	0.003921568627451f


	#define DDIFF(a,b) (((a) < (b)) ? ((b) - (a)) : ((a) - (b)))
	#define DMIN(a,b) ((a) < (b) ? (a) : (b))
	#define DMAX(a,b) ((a) > (b) ? (a) : (b))
	#define DABS(a) ((a) > 0 ? (a) : -(a))
	#define DCLAMP(a, min, max) ((a) < (min) ? (min) : ((a) > (max) ? (max) : (a)))
	#define DLERP(min, max, t) ((min) + ((max) - (min)) * (t))

	// Vector macros.
	
	// MAKE SURE NOT TO HAVE DEST BE SOURCE1 OR SOURCE2 or it'll screw up!
	#define VEC_CROSS(dest, v1, v2) \
		{\
		(dest).x = ((v2).y*(v1).z - (v2).z*(v1).y);\
		(dest).y = ((v2).z*(v1).x - (v2).x*(v1).z);\
		(dest).z = ((v2).x*(v1).y - (v2).y*(v1).x);\
		}
	
	
	#define VEC_ADD(d, v1, v2) \
		{\
		(d).x = (v1).x + (v2).x;\
		(d).y = (v1).y + (v2).y;\
		(d).z = (v1).z + (v2).z;\
		}

	#define VEC_ADDSCALED(d, v1, v2, s) \
		{\
		(d).x = (v1).x + ((v2).x * (s));\
		(d).y = (v1).y + ((v2).y * (s));\
		(d).z = (v1).z + ((v2).z * (s));\
		}

	#define VEC_SUB(d, v1, v2) \
		{\
		(d).x = (v1).x - (v2).x;\
		(d).y = (v1).y - (v2).y;\
		(d).z = (v1).z - (v2).z;\
		}

	#define VEC_MUL(d, v1, v2) \
		{\
		(d).x = (v1).x * (v2).x;\
		(d).y = (v1).y * (v2).y;\
		(d).z = (v1).z * (v2).z;\
		}

	#define VEC_DIV(d, v1, v2) \
		{\
		(d).x = (v1).x / (v2).x;\
		(d).y = (v1).y / (v2).y;\
		(d).z = (v1).z / (v2).z;\
		}

	#define VEC_MULSCALAR(d, v1, s) \
		{\
		(d).x = (v1).x * (s); \
		(d).y = (v1).y * (s); \
		(d).z = (v1).z * (s); \
		}

	#define VEC_DIVSCALAR(d, v1, s) \
		{\
		(d).x = (v1).x / (s); \
		(d).y = (v1).y / (s); \
		(d).z = (v1).z / (s);\
		}

	#define VEC_LERP(d, v1, v2, t) \
		{\
		(d).x = (v1).x + ((v2).x - (v1).x) * t;\
		(d).y = (v1).y + ((v2).y - (v1).y) * t;\
		(d).z = (v1).z + ((v2).z - (v1).z) * t;\
		}

	#define VEC_CLAMP(v, a, b) \
	{\
		(v).x = DCLAMP((v).x, a, b);\
		(v).y = DCLAMP((v).y, a, b);\
		(v).z = DCLAMP((v).z, a, b);\
	}

	#define VEC_MIN(v, a, b) \
	{\
		(v).x = DMIN((a).x, (b).x);\
		(v).y = DMIN((a).y, (b).y);\
		(v).z = DMIN((a).z, (b).z);\
	}

	#define VEC_MAX(v, a, b) \
	{\
		(v).x = DMAX((a).x, (b).x);\
		(v).y = DMAX((a).y, (b).y);\
		(v).z = DMAX((a).z, (b).z);\
	}

	#define VEC_NEGATE(d, s) \
		{\
		(d).x = -(s).x; \
		(d).y = -(s).y; \
		(d).z = -(s).z; \
		}

	// Dot product of 2 vectors.
	#define VEC_DOT(v1, v2) ( (v1).x*(v2).x + (v1).y*(v2).y + (v1).z*(v2).z )

	// Expand the vector for function calls..
	#define EXPANDVEC(vec) (vec).x, (vec).y, (vec).z
	
	// Distance between 2 vectors.
	#define VEC_DISTSQR(v1, v2) ( \
		((v1).x-(v2).x) * ((v1).x-(v2).x) + \
		((v1).y-(v2).y) * ((v1).y-(v2).y) + \
		((v1).z-(v2).z) * ((v1).z-(v2).z) )

	#define VEC_DIST(v1, v2) ((float)sqrt(VEC_DISTSQR(v1, v2)))

	#define VEC_MAGSQR(v) ((v).x*(v).x + (v).y*(v).y + (v).z*(v).z)
	#define VEC_MAG(v) ((float)sqrt(VEC_MAGSQR(v)))

	#define VEC_INVMAG(v) (1.0f / VEC_MAG(v))

	#define VEC_NORM(v) \
		{\
			float __temp_normalizer_____;\
			__temp_normalizer_____ = 1.0f / VEC_MAG(v);\
			(v).x *= __temp_normalizer_____;\
			(v).y *= __temp_normalizer_____;\
			(v).z *= __temp_normalizer_____;\
		}

	#define VEC_COPY(dest, src) \
		{\
			(dest).x = (src).x; (dest).y = (src).y; (dest).z = (src).z;\
		}
	
	#define VEC_SET(v, vx, vy, vz) \
		{\
			(v).x = (float)(vx); (v).y = (float)(vy); (v).z = (float)(vz);\
		}
	
	#define VEC_EXPAND(v) (v).x, (v).y, (v).z
	#define VEC_INIT(v) (v).x = (v).y = (v).z = 0.0f;
	#define VEC_INDEX(v, i) (((float*)&(v))[i])
	
	#define ROT_COPY(dest, src) \
		{\
		VEC_COPY((dest).m_Vec, (src).m_Vec);\
		(dest).m_Spin = (src).m_Spin;\
		}

	#define PLANE_COPY(dest, src) \
		{\
		VEC_COPY((dest).m_Normal, (src).m_Normal)\
		(dest).m_Dist = (src).m_Dist;\
		}

	#define PLANE_SET(plane, _x, _y, _z, _dist) \
	{\
		(plane).m_Normal.x = (_x);\
		(plane).m_Normal.y = (_y);\
		(plane).m_Normal.z = (_z);\
		(plane).m_Dist = (_dist);\
	}

	#define MAT_COPY(dest, src) \
		{\
		(dest).m[0][0] = (src).m[0][0];\
		(dest).m[0][1] = (src).m[0][1];\
		(dest).m[0][2] = (src).m[0][2];\
		(dest).m[0][3] = (src).m[0][3];\
													 \
		(dest).m[1][0] = (src).m[1][0];\
		(dest).m[1][1] = (src).m[1][1];\
		(dest).m[1][2] = (src).m[1][2];\
		(dest).m[1][3] = (src).m[1][3];\
													 \
		(dest).m[2][0] = (src).m[2][0];\
		(dest).m[2][1] = (src).m[2][1];\
		(dest).m[2][2] = (src).m[2][2];\
		(dest).m[2][3] = (src).m[2][3];\
													 \
		(dest).m[3][0] = (src).m[3][0];\
		(dest).m[3][1] = (src).m[3][1];\
		(dest).m[3][2] = (src).m[3][2];\
		(dest).m[3][3] = (src).m[3][3];\
		}

	#define MAT_SET(dest, \
		e00, e01, e02, e03,\
		e10, e11, e12, e13,\
		e20, e21, e22, e23,\
		e30, e31, e32, e33)\
		{\
		(dest).m[0][0] = e00; (dest).m[0][1] = e01; (dest).m[0][2] = e02; (dest).m[0][3] = e03;\
		(dest).m[1][0] = e10; (dest).m[1][1] = e11; (dest).m[1][2] = e12; (dest).m[1][3] = e13;\
		(dest).m[2][0] = e20; (dest).m[2][1] = e21; (dest).m[2][2] = e22; (dest).m[2][3] = e23;\
		(dest).m[3][0] = e30; (dest).m[3][1] = e31; (dest).m[3][2] = e32; (dest).m[3][3] = e33;\
		}


	// Get the distance from a point to a plane.
	#define DIST_TO_PLANE(vec, plane) ( VEC_DOT((plane).m_Normal, (vec)) - (plane).m_Dist )

	// Initializes the rotation to look down the positive Z axis.
	#define ROT_INIT(r) \
		{\
		(r).m_Vec.x = (r).m_Vec.y = (r).m_Vec.z = 0.0f;\
		(r).m_Spin = 1.0f;\
		}


	// Used for resizing arrays.. mostly for internal DirectEngine stuff.
	#define GENERIC_REMOVE(theArray, iIndex, nElements) \
		ASSERT(nElements > 0);\
		memmove(&theArray[iIndex], &theArray[iIndex+1], sizeof(theArray[0]) * (nElements-iIndex-1));\
		nElements--;

	#define GENERIC_INSERT(type, theArray, nElements, iIndex, value) \
		{\
			type *pNewArray = (type*)malloc(sizeof(type) * (nElements+1));\
			memcpy(pNewArray, theArray, nElements * sizeof(type));\
			memmove(&pNewArray[iIndex+1], &pNewArray[iIndex], sizeof(type)*(nElements-iIndex));\
			memcpy(&pNewArray[iIndex], &value, sizeof(type));\
			free(theArray);\
			theArray = pNewArray;\
			++nElements;\
		}
				
	#define GENERIC_APPEND(type, theArray, nElements, value) \
		GENERIC_INSERT(type, theArray, nElements, nElements, value);
				
	#define DTOCVEC(_vec) (*((CVector*)&(_vec)))
	#define CTODVEC(_vec) (*((DVector*)&(_vec)))




	typedef struct
	{
		float x, y, z;
	} DVector;

	typedef struct DRotation_t
	{
		// <vector>, spin
		DVector	m_Vec;
		float	m_Spin;
	} DRotation;

	typedef struct DPlane_t
	{
		DVector	m_Normal;
		float	m_Dist;
	} DPlane;

	typedef struct DFloatPt_t
	{
		float	x, y;
	} DFloatPt;

	typedef struct DWarpPt_t
	{
		float source_x, source_y;
		float dest_x, dest_y;
	} DWarpPt;

	typedef struct DRect_t
	{
		int left, top, right, bottom;
	} DRect;

	typedef struct
	{
		char **argv;
		int argc;
	} ArgList;

	// Surface data.
	typedef struct SurfaceData_t
	{
		DVector O, P, Q; // Texture vectors.
		unsigned char m_EffectParam; // Effect parameter.
	} SurfaceData;

	// Sky definition.
	typedef struct SkyDef_t
	{
		DVector	m_Min, m_Max;			// Box corners (in world coordinates).
		DVector	m_ViewMin, m_ViewMax;	// View min and max.  The viewer's position
										// in the main world is squished into this
										// box so you can get a tiny amount of parallax.
	} SkyDef;


	#include "matrix_ops.h"
	#include "dlink.h"



	typedef DDWORD HMODELANIM;
	typedef void (*ConsoleProgramFn)(int argc, char **argv);


	// Object flags.
	#define FLAG_VISIBLE			(1<<0)
	#define FLAG_SHADOW				(1<<1)
	#define FLAG_MODELSKIN			(1<<2)
	
	#define FLAG_CASTSHADOWS		(1<<3)	// Should this light cast shadows (slower)?
	#define FLAG_ROTATEABLESPRITE	(1<<3)	// Sprites only.
	#define FLAG_MODELGOURAUDSHADE	(1<<3)	// Models only.
	#define FLAG_UPDATEUNSEEN		(1<<3)	// Particle systems only.
											// If this is set, the engine will update
											// particles even when they're invisible.
											// You should check FLAG_WASDRAWN
											// on any particle systems you're iterating
											// over so you don't update invisible ones.
	
	#define FLAG_SOLIDLIGHT			(1<<4)	// Use the 'fastlight' method for this light.
	#define FLAG_SPRITECHROMAKEY	(1<<4)
	#define FLAG_MODELWIREFRAME		(1<<4)
	#define FLAG_WASDRAWN			(1<<4)	// The engine sets this if a particle system
											// or PolyGrid was drawn.  You can use this to 
											// determine whether or not to do some expensive
											// calculations on it.

	#define FLAG_GLOWSPRITE			(1<<5)	// Shrinks the sprite as the viewer gets nearer.
	#define FLAG_ONLYLIGHTWORLD		(1<<5)	// Lights only - tells it to only light the world.
	#define FLAG_NOLIGHT			(1<<5)	// Just use the object's color and global light scale.
											// (Don't affect by area or by dynamic lights).
	
	#define FLAG_SPRITEBIAS				(1<<6)	// Biases the Z towards the view so a sprite doesn't clip as much.
	#define FLAG_DONTLIGHTBACKFACING	(1<<6)	// Lights only - don't light backfacing polies.
	#define FLAG_REALLYCLOSE			(1<<6)	// Used for models really close to the view (like PV weapons).
												// Must use tricks so it doesn't clip off.

	#define FLAG_SPRITE_NOZ				(1<<7)	// Disable Z read/write on sprite (good for lens flares).
												// These sprites must not be chromakeyed.

	#define FLAG_FULLPOSITIONRES		(1<<8)	// LT normally uses a position resolution of 
												// 256k units throughout the level to reduce
												// packet size.  Some things must be exact 
												// (like some WorldModels) so this will 
												// enlarge the packets for better accuracy.
	
	// Which flags the client knows about.
	#define CLIENT_FLAGMASK			(FLAG_VISIBLE|FLAG_SHADOW|FLAG_MODELSKIN|\
									FLAG_ROTATEABLESPRITE|FLAG_SPRITECHROMAKEY|\
									FLAG_NOLIGHT|FLAG_REALLYCLOSE|FLAG_SPRITE_NOZ|\
									FLAG_FULLPOSITIONRES)

	// Server only flags.
	#define FLAG_SOLID				(1<<12)	// Object can't go thru other solid objects.
	#define FLAG_TOUCH_NOTIFY		(1<<13)	// Gets touch notification.
	#define FLAG_GRAVITY			(1<<14)	// Gravity is applied.
	#define FLAG_STAIRSTEP			(1<<15)	// Steps up stairs.
	#define FLAG_MODELKEYS			(1<<16)	// The object won't get get MID_MODELSTRINGKEY messages unless
											// it sets this flag.
	#define FLAG_KEEPALIVE			(1<<17)	// Save and restore this object when switching worlds.
	#define FLAG_GOTHRUWORLD		(1<<18) // Object can pass through world
	#define FLAG_RAYHIT				(1<<19) // Object is hit by raycasts.
	#define FLAG_DONTFOLLOWSTANDING	(1<<20) // Dont follow the object this object is standing on.
	
	// Internal flags.  Descriptions are there to help the DE developers remember what
	// they're there for, NOT for general use!
	#define FLAG_INTERNAL1			(1<<21)	// (Did the renderer see the object).
	#define FLAG_INTERNAL2			(1<<22)	// (Used by ClientDE::FindObjectsInSphere).
	#define FLAG_LASTFLAG			(1<<22) 



	// Different object types.  Some can only be created on the client.
	#define OT_NORMAL			0	// Invisible object.  Note, client's aren't told about
									// these when they're created on the server! 
	#define OT_MODEL			1	// Model object.
	#define OT_WORLDMODEL		2	// WorldModel.
	#define OT_SPRITE			3	// Sprite.
	#define OT_LIGHT			4	// Dynamic light.
	
	#define OT_CAMERA			5	// Camera.
	#define OT_PARTICLESYSTEM	6	// Particle system.
	#define OT_POLYGRID			7	// Poly grid.
	#define OT_LINESYSTEM		8	// Line system.
	#define OT_CONTAINER		9	// Container.
	#define NUM_OBJECTTYPES		10

	// Size defines used for Parse functions
	// Use these to size your argument buffer and argument pointer
	// buffer accordingly.
	#define PARSE_MAXARGS		30
	#define PARSE_MAXARGLEN		80


#endif 

