

#ifndef __CLIENT_DE_H__
#define __CLIENT_DE_H__


	#include "basedefs_de.h"
	#include "clientshell_de.h"


	typedef void* HCONSOLEVAR;


	// Particle system flags. Each one slows it down a little more
	// except for PS_NEVERDIE which speeds it up..
	#define PS_BOUNCE		1	// Do they bounce?
	#define PS_SHADOWS		2	// Enable shadows.
	#define PS_NEVERDIE		4	// The particles never die (and it doesn't have to check).
	#define PS_DUMB			8	// The engine leaves the system alone.. you must
								// update and move them.


	
	// The particle structure for particle systems.
	typedef struct DEParticle_t
	{
		DVector m_Velocity;
		DVector	m_Color;	// Particle colors are 0-255.
		struct DEParticle_t *m_pNext;
	} DEParticle;

	
	// The line structure for line system.
	typedef struct DELinePt_t
	{
		DVector		m_Pos;
		float		r, g, b, a;	// Values 0-1.
	} DELinePt;

	typedef struct DELine_t
	{
		DELinePt	m_Points[2];
	} DELine;

	DEFINE_HANDLE_TYPE(HDELINE)



	// ------------------------------------------------------------------ //
	// Intersection tests
	// ------------------------------------------------------------------ //

	// Pass this in to the IntersectSegment routine.
	typedef struct ClientIntersectQuery_t
	{
		DDWORD			m_Flags;

		// Start and end points.
		DVector			m_From;
		DVector			m_To;

		// Only used for CastRay.. this is the direction the ray should be cast in.
		// This doesn't need to be normalized.
		DVector			m_Direction;
	} ClientIntersectQuery;

	
	typedef struct ClientIntersectInfo_t
	{
		// Point of intersection.
		DVector	m_Point;

		// Plane of intersection.
		DPlane	m_Plane;

		// Poly it hit.
		HPOLY	m_hPoly;
	} ClientIntersectInfo;


	// Used in GetFileList/FreeFileList.
	#define TYPE_FILE		1
	#define TYPE_DIRECTORY	2
	
	typedef struct FileEntry_t
	{
		int			m_Type;				// Directory or file?
		char		*m_pBaseFilename;	// pic1.pcx
		char		*m_pFullFilename;	// interface/bitmaps/pic1.pcx
		struct FileEntry_t	*m_pNext;
	} FileEntry;



	// ------------------------------------------------------------------ //
	// Client-side object flags.
	// ------------------------------------------------------------------ //

	// The engine calls ClientShell::OnObjectRemove() when the object goes away.
	#define CF_NOTIFYREMOVE		1

	
	// ------------------------------------------------------------------ //
	// Use this to start a game.
	// ------------------------------------------------------------------ //

	#define STARTGAME_HOST		0	// Start a world and host it (using dialogs).
	#define STARTGAME_HOSTTCP	1	// Start a world and host on TCP/IP.

	#define STARTGAME_CLIENT	2	// Connect to a server using dialogs.
	#define STARTGAME_CLIENTTCP	3	// Connect to the first TCP/IP game it can find
									// at m_pTCPAddress.
	
	#define STARTGAME_NORMAL	4	// Start a normal game.
	#define GAMEMODE_NONE		5	// (Used for GetGameMode, means we're not 
									// running a world or on a server yet).

	typedef struct StartGameRequest_t
	{
		int		m_Type;
		char	*m_pWorldName;
		char	*m_pTCPAddress;	// TCP/IP address, if any.
		char	*m_pRecordFilename;	// Filename to record into, if any (null if you don't want to record).
	} StartGameRequest;


	// ---------------------------------------------------------------------- //
	// ---------------------------------------------------------------------- //
	// Sound effects.
	// 
	// Structure filled out by code playing sound.  Sound is destroyed in
	// the following ways:
	//
	//		not looping, no handle
	//			not attached
	//				Client destroys when sound finished.
	//			attached
	//				Client destroys when object destroyed.
	//
	//		handle returned
	//			Client does not destroy, must be destroyed by creator.
	//
	// The type of sounds are:  ambient, local and 3D.  The flags controlling
	// these types are mutually exclusive.  An ambient sound will have distance
	// effects of rolloff, but no orientation effects.  A local sound will have
	// no orientation or distance effects and will be as if the sound was played
	// inside the player's head.  The 3d sound will have distance, orientation
	// and doppler effects.
	//
	// Sounds are played from the beginning once the message reaches the client.
	// If it is important that the playback be synchronized with the server's 
	// beginning time, then set PLAYSOUND_TIMESYNC.  Normally, this is not
	// that important.  The server will use it internally if a client comes in
	// after a sound has already been played.  It does this on looping sounds.
	//
	// ---------------------------------------------------------------------- //

	#define PLAYSOUND_LOCAL			0x0000	// Play sound locally (inside head)
	#define PLAYSOUND_AMBIENT		0x0001	// Play sound as ambient sound.
	#define PLAYSOUND_3D			0x0002	// Play sound as 3D sound.
	#define PLAYSOUND_LOOP			0x0004	// Loop the sound.
	#define PLAYSOUND_ATTACHED		0x0008	// Attached to object m_hObject
	#define PLAYSOUND_GETHANDLE		0x0010	// Handle requested
	#define PLAYSOUND_TIME			0x0020	// Server must time sound
	#define PLAYSOUND_CTRL_VOL		0x0040	// Control volume m_nVolume
	#define PLAYSOUND_CTRL_FREQ		0x0080	// Control frequency m_wFrequency
	#define PLAYSOUND_CTRL_PAN		0x0100	// Control pan m_wPan
	#define PLAYSOUND_CTRL_ORIENT	0x0200	// Control orientation.  Must have PLAYSOUND_3D and PLAYSOUND_CTRL_VOL also.
	#define PLAYSOUND_CLIENT		0x0400	// Client side sound
	#define PLAYSOUND_TIMESYNC		0x0800	// Playback synchronized with server clock
	#define PLAYSOUND_FILESTREAM	0x1000	// Stream the file

	typedef struct PlaySoundInfo_t
	{
		// PLAYSOUND flags.
		DDWORD		m_dwFlags;
	
		// File name of sound
		char 		m_szSoundName[_MAX_PATH];

		// Object ID sound is attached to.
		// Only needed if PLAYSOUND_ATTACHED set
		D_WORD		m_wObjectID;

		// Handle of sound
		// Filled by PlaySound if PLAYSOUND_GETHANDLE set
		HSOUNDDE	m_hSound;

		// Voice type
		// Value must be 0 through InitSoundIno::m_nNumVoiceTypes - 1
		unsigned char m_nVoiceType;

		// Voice priority within voice type m_nVoiceType
		// 0 is lowest priority
		unsigned char m_nPriority;

		// Maximum radius of sound.  No sound outside this.
		// Only needed if PLAYSOUND_3D or PLAYSOUND_AMBIENT
		float		m_fOuterRadius;
		
		// Inner radius of sound.  Sound at maximum volume inside this radius
		// Only needed if PLAYSOUND_3D or PLAYSOUND_AMBIENT
		float		m_fInnerRadius;
		
		// Volume of sound in percent [0,100].
		// Only needed if PLAYSOUND_CTRL_VOL set, otherwise defaults 100
		// If PLAYSOUND_3D set, then this is the inside cone volume
		DBYTE		m_nVolume;

		// Frequency adjustment multiplier (0.0,MAX_FLOAT)
		// Only needed if PLAYSOUND_LOCAL and PLAYSOUND_CTRL_FREQ set
		// If PLAYSOUND_CTRL_FREQ clear, then defaults 1.0
		float		m_fFrequencyScale;

		// Pan in percent [-100(left),100(right)]
		// Only needed if PLAYSOUND_LOCAL and PLAYSOUND_CTRL_FREQ set
		// If PLAYSOUND_CTRL_PAN clear, then defaults 0
		signed char	m_iPan;

		// Position of sound.
		// Only needed if PLAYSOUND_AMBIENT and PLAYSOUND_3D set
		// Only needed if PLAYSOUND_ATTACHED cleared
		DVector		m_vPosition;

		// Forward orientation of sound.
		// Only needed if PLAYSOUND_3D set
		// Only needed if PLAYSOUND_ATTACHED cleared.  If attached, uses object's orientation.
		// Only needed if PLAYSOUND_CTRL_ORIENT set
		DVector		m_vOrientation;

		// Sound cone arc length in degrees [0,360]
		// Only needed if PLAYSOUND_3D set
		// Must have PLAYSOUND_CTRL_VOL set, otherwise defaults 360
		// Only needed if PLAYSOUND_CTRL_ORIENT set
		D_WORD		m_wConeAngle;

		// Outside of cone volume scale in percent [0,100]
		// Only needed if PLAYSOUND_3D set
		// Must have PLAYSOUND_CTRL_VOL set, otherwise defaults 100
		// Only needed if PLAYSOUND_CTRL_ORIENT set
		DBYTE		m_nOutsideConeVolume;

	} PlaySoundInfo;

	#define PLAYSOUNDINFO_COPY( dest, src ) \
		memcpy(( void * )&( dest ), ( void * )&( src ), sizeof( PlaySoundInfo ));

	#define PLAYSOUNDINFO_INIT(x) memset(&x, 0, sizeof(x));\


	//  Music info

	#define MUSIC_IMMEDIATE			0
	#define MUSIC_NEXTMEASURE		1
	#define MUSIC_NEXTSONG			2
	#define MUSIC_QUEUE				3


	// Used in AddSurfaceEffect.
	typedef struct SurfaceEffectDesc_t
	{
		char	*m_pName;  // You don't have to allocate this..
		
		// If you return NULL from here, the effect won't be initialized.
		void*	(*InitEffect)(struct SurfaceData_t *pSurfaceData, int argc, char **argv);
		int		(*UpdateEffect)(struct SurfaceData_t *pSurfaceData, void *pData);	// pData is what you returned from InitEffect.
												// Return 1 if you changed the surface.
		void	(*TermEffect)(void *pData);
	} SurfaceEffectDesc;


	// ------------------------------------------------------------------ //
	// Model hook stuff.
	// ------------------------------------------------------------------ //

	typedef void (*ModelHookFn)(struct ModelHookData_t *pData, void *pUser);



	// ------------------------------------------------------------------ //
	// ClientDE interface. This is what the ClientShells and special
	// effect plugins use to do the interface and client objects for the game.
	// ------------------------------------------------------------------ //

	typedef struct ClientDE_t
	{

		// Connection stuff.

			// Tries to start the game in the mode you specify.
			// This will keep the existing game running until it successfully starts
			// the new game.. ie: if there's an error starting the new world, it
			// will still leave the current one running and put up an error message.
			DRESULT		(*StartGame)(StartGameRequest *pRequest);

			// Find out what the current game mode is (how we started the last
			// game with StartGame or thru the console).  Fills in mode with 
			// one of the STARTGAME_ defines above.
			DRESULT		(*GetGameMode)(int *mode);

			// Gets the local client ID.
			// Returns DE_NOTCONNECTED if we're not on a server or haven't
			// gotten a client ID yet.
			DRESULT		(*GetLocalClientID)(DDWORD *pID);

			DBOOL		(*IsConnected)();	// Are we on a server currently?

			// Disconnect from the server we're on (if any).
			// NOTE: this will destroy any client-side objects you've created!
			void		(*Disconnect)();

			// Shuts down the app (not right away, but after the current update).
			void		(*Shutdown)();

			// Returns DE_NOTINWORLD if no world is loaded, or DE_INSIDE if the point
			// is in the world or DE_OUTSIDE if the point is not in the world.
			DRESULT		(*GetPointStatus)(DVector *pPoint);

		
		// Renderer management.

			// Get a (NULL-terminated) list of supported render modes.
			RMode*		(*GetRenderModes)();
			void		(*RelinquishRenderModes)(RMode *pModes);
			
			// Fills in the current render mode.
			DRESULT		(*GetRenderMode)(RMode *pMode);

			// Goes into the requested render mode.  If it can't, it tries
			// to go back into the render mode it was in previously.  If
			// it can't do that, it shuts down with an error.
			DBOOL		(*SetRenderMode)(RMode *pMode);


		// File management.
		
			// Get a list of files/directories in a directory (pass in "" to start with).
			// The list is a noncircular linked list with DNULL terminating it.
			FileEntry*	(*GetFileList)(char *pDirName);
			
			// Use FreeFileList when you're done with each list you get.
			void		(*FreeFileList)(FileEntry *pHead);


		// Interface stuff.

			void		(*GetAxisOffsets)(DFLOAT *offsets);

			void		(*PlayJoystickEffect)(char *pEffectName, float x, float y);
		

		// Music functions.

			DBOOL		(*InitMusic)( char *szMusicDLL );

			// Sets the directory to find music data files.
			// IMA only.
			DBOOL		(*SetMusicDirectory)( char *szMusicDirectory );

			// Downloads DLS.
			// szDLSFile is the Downloadable Synthizer file containing the waves for 
			// instruments.  szStyleFile is a style file, that is a compainion to the 
			// DLS file.  It must contain bands that define the program changes for
			// each instrument in DLS...
			// IMA only.
			DBOOL		(*InitInstruments)( char *szDLSFile, char *szStyleFile );

			// Cache in a song file.
			DBOOL		(*LoadSong)(char *szSong);

			// Remove all the songs from memory...
			void		(*DestroyAllSongs)();

			// Plays a playlist.  All songs will be played until end of list.  
			// Arguments:
			//		szPlayList - Name of playlist.
			//		szTransition - Name of transition song to play before playlist starts.  IMA only.
			//		bLoop - Loop the list.
			//		dwBoundaryFlags - Can be MUSIC_IMMEDIATE, MUSIC_MEASURE, MUSIC_SONG, MUSIC_QUEUE.  Valid for IMA only.
			DBOOL		(*PlayList)( char *szPlayList, char *szTransition, DBOOL bLoop, DDWORD dwBoundaryFlags );

			// Plays a musical break.
			// IMA only.
			// Arguments:
			//		szSong - Name of song.
			//		dwBoundaryFlags - Can be MUSIC_IMMEDIATE, MUSIC_MEASURE, MUSIC_SONG, MUSIC_QUEUE.  Valid for IMA only.
			DBOOL		(*PlayBreak)( char *szSong, DDWORD dwBoundaryFlags );

			// Stops music.  
			// Arguments:
			//		dwBoundaryFlags - Can be MUSIC_IMMEDIATE, MUSIC_MEASURE, MUSIC_SONG, MUSIC_QUEUE.  Valid for IMA only.
			void		(*StopMusic)( DDWORD dwBoundaryFlags );

			// Pause music.  Can be resumed...
			DBOOL		(*PauseMusic)();

			// Resume music...
			DBOOL		(*ResumeMusic)();

			// Create and add to playlists....
			// Arguments:
			//		szPlayList - Name of playlist.
			//		szSong - Name of song to add.
			DBOOL		(*AddSongToPlayList)(char *szPlayList, char *szSong);

			// Removes a whole playlist...
			// Arguments:
			//		szPlayList - Name of playlist.
			void		(*DeletePlayList)(char *szPlayList);

			// Plays a motif.
			// Ima only.
			// Arguments:
			//		szMotif - name of motif.
			//		bLoop - Loop the motif.
			DBOOL		(*PlayMotif)(char *szMotif, DBOOL bLoop);

			// Stops a motif for IMA.
			// IMA only.
			// Arguments:
			//		szMotif - name of motif.
			void		(*StopMotif)(char *szMotif);

			// Volume range is 0-off, 100-full.
			short		(*GetMusicVolume)();
			void		(*SetMusicVolume)( short wVolume );


		// Sound functions.

			// Volume controls between 0 - 100
			unsigned short	( *GetSoundVolume )();
			void			( *SetSoundVolume )( unsigned short nVolume );

			// These PlaySound functions allow the client shell to initiate and control sounds
			// without going through the server.  These sounds will only be played 
			// on the client initiating them.
			// Because these PlaySound functions create sound that are guaranteed not to come from
			// the server, the m_hSound variable is unused.  These functions will fill the variable
			// with the handle to the local sound.  If you set the PLAYSOUND_GETHANDLE flag, then
			// the sound will not be automatically deleted when the sound is done playing.  You
			// must call KillSound.  The handle is also the return value of each playsound function.  
			// You can then use this handle for the IsDone and KillSound functions.

				// Play a sound with full control
				// Arguments:
				//		pPlaySoundInfo - sound control structure
				// Return:
				//		Handle to the client only sound.
				HSOUNDDE	(*PlaySound)( PlaySoundInfo *pPlaySoundInfo );

				// Play sound without effects (inside players head), sound removed when finished playing.
				// Arguments:
				//		pSoundName - relative path to sound name including ext.
				//		bGetHandle - Set the PLAYSOUND_GETHANDLE flag
				// Return:
				//		Handle to the client only sound.
				HSOUNDDE	(*PlaySoundLocal)(char *pSoundName, DBOOL bGetHandle, unsigned char nType, unsigned char nPriority  );

				// Check if sound finished playing or if object it was attached to was removed.
				// Arguments:
				//		pSoundHandle - handle to client only sound
				DBOOL	(*IsDone)( HSOUNDDE pSoundHandle );

				// Kill a sound.
				// Arguments:
				//		pSoundHandle - handle to client only sound
				void	(*KillSound)( HSOUNDDE pSoundHandle );


		// Video functions.  These functions use Smacker movies.

			// DE_OK, VIDEO_ERROR.
			// flags is a combination of the PLAYBACK_ flags in de_codes.h.
			DRESULT		(*StartVideo)(char *pFilename, DDWORD flags);
			DRESULT		(*StopVideo)();
			
			// VIDEO_PLAYING, VIDEO_NOTPLAYING.
			DRESULT		(*IsVideoPlaying)();


		// String functions.  Strings are reference counted objects that cannot
		// be manipulated.  When you create one with FormatString or CreateString,
		// the reference count is 1.  When you copy a string with CopyString, the
		// reference count is incremented.  When you free one with FreeString,
		// it decrements the reference count.. when the reference count goes to
		// zero, it deletes the string.  If you forget to free up any strings, 
		// DirectEngine will spit out a message telling you about it..

			// In Windows, messageCode comes from your resource DLL.  The messages
			// need to be formatted with %1!s! %2!s! (the number is the argument 
			// number and the !s! says its a string).  You can also use !f! and !d!
			// for floating point and whole number.
			HSTRING		(*FormatString)(int messageCode, ...);
			
			// Copy a string.. much more efficient than CreateString().
			HSTRING		(*CopyString)(HSTRING hString);
			HSTRING		(*CreateString)(char *pString);
			void		(*FreeString)(HSTRING hString);

			DBOOL		(*CompareStrings)(HSTRING hString1, HSTRING hString2);
			DBOOL		(*CompareStringsUpper)(HSTRING hString1, HSTRING hString2);

			// Get the string's data.. you really should only use this for strings
			// that you stored off and want to pass to the engine for a filename 
			// or something..  Most strings could be in some format other than ANSI.
			char*		(*GetStringData)(HSTRING hString);


		// Intersections.

			// Intersect a line segment.. (used to be raycast, but line segments are WAY faster).
			// Returns TRUE and fills in pInfo if it hit something.
			DBOOL		(*IntersectSegment)(ClientIntersectQuery *pQuery, ClientIntersectInfo *pInfo);

			// Same as IntersectSegment, except for it casts a ray from pQuery->m_From
			// in the direction of pQuery->m_Dir.
			DBOOL		(*CastRay)(ClientIntersectQuery *pQuery, ClientIntersectInfo *pInfo);

			// Find objects in a sphere.  This is only based on their centerpoints, not
			// on their dimensions or anything.  inObjects is filled in and nOutObjects 
			// is set to the number of objects filled in.  nFound might be larger if
			// it found more objects than nInObjects.
			DRESULT		(*FindObjectsInSphere)(DVector *pCenter, float radius, 
				HLOCALOBJ *inObjects, DDWORD nInObjects, DDWORD *nOutObjects, DDWORD *nFound);


		// Fonts..

			// Creates the closest font it can to the one you asked for.
			HDEFONT		(*CreateFont)(char *pFontName, int width, int height, 
				DBOOL bItalic, DBOOL bUnderline, DBOOL bBold);

			void		(*DeleteFont)(HDEFONT hFont);


		// Colors..

			// r, g, and b go from 0 to 1.
			// bTransparent is only used when the function specifies so.
			// Note: anywhere you can pass a color, if you pass in NULL, it'll use black.
			HDECOLOR	(*CreateColor)(float r, float g, float b, DBOOL bTransparent);
			void		(*DeleteColor)(HDECOLOR hColor);

			// Just for convenience.  You don't have to create/delete these colors,
			// and they're always around!
			HDECOLOR	(*SetupColor1)(float r, float g, float b, DBOOL bTransparent);
			HDECOLOR	(*SetupColor2)(float r, float g, float b, DBOOL bTransparent);

		
		// Surface management.
		// Note:  All the rectangles you specify in here do not include the right and bottom
		//        edges.  ie: If you draw a rectangle (0, 1, 0, 1), it'll draw 1 pixel
		//        instead of 4 pixels. 

		// Note:  Try to use the screen surface as little as possible.  Using the screen
		//        surface stalls ALL asynchronous rendering performance and can cut framerate
		//        in half on some cards.  That's not to say don't use them in the interface,
		//        just don't use them for things that are always there like frag counts, etc..
		//        The only functions that don't stall async performance (feel free to use
		//        these regularly) are:
		//           DrawSurfaceToSurface
		//           DrawSurfaceToSurfaceTransparent
		//           ScaleSurfaceToSurface
		//           ScaleSurfaceToSurfaceTransparent
		
			HSURFACE	(*GetScreenSurface)();

			// Creates a surface sized to the dimensions of the bitmap.
			// The bitmap is an 8-bit (you can use any palette you want..) PCX file.
			// So pBitmapName might look like "interface/bitmaps/menu1.pcx".
			HSURFACE	(*CreateSurfaceFromBitmap)(char *pBitmapName);
			
			// Creates a surface just large enough for the string.
			// You can make the surface a little larger with extraPixelsX and extraPixelsY.
			HSURFACE	(*CreateSurfaceFromString)(HDEFONT hFont, HSTRING hString,
				HDECOLOR hForeColor, HDECOLOR hBackColor, 
				int extraPixelsX, int extraPixelsY);
			
			// Create a plain old surface.
			HSURFACE	(*CreateSurface)(DDWORD width, DDWORD height);
			
			DRESULT		(*DeleteSurface)(HSURFACE hSurface);

			// Attach whatever user data you want to a surface.
			void*		(*GetSurfaceUserData)(HSURFACE hSurface);
			void		(*SetSurfaceUserData)(HSURFACE hSurface, void *pUserData);

			// Access the pixels (SLOW).
			DRESULT		(*GetPixel)(HSURFACE hSurface, DDWORD x, DDWORD y, HDECOLOR *color);
			DRESULT		(*SetPixel)(HSURFACE hSurface, DDWORD x, DDWORD y, HDECOLOR color);
			
			// Gets the dimensions that this string would take up.
			void		(*GetStringDimensions)(HDEFONT hFont, HSTRING hString, int *sizeX, int *sizeY);

			// Draws the string into the rectangle..
			void		(*DrawStringToSurface)(HSURFACE hDest, HDEFONT hFont, HSTRING hString, 
				DRect *pRect, HDECOLOR hForeColor, HDECOLOR hBackColor);

			// You can pass in NULL for pWidth and pHeight if you want.
			void		(*GetSurfaceDims)(HSURFACE hSurf, DDWORD *pWidth, DDWORD *pHeight);
			
			// Draw a bitmap to a surface..
			DBOOL		(*DrawBitmapToSurface)(HSURFACE hDest, char *pSourceBitmapName, 
				DRect *pSrcRect, int destX, int destY);

			// Draws hSrc to hDest like a normal transparent blit, but tiles hMask's surface
			// into the nontransparent pixels of hSrc (can you say airbrushed text?)
			// You can't have a mask texture larger than 256x256, and the mask must be
			// a power of 2.
			void		(*DrawSurfaceMasked)(HSURFACE hDest, HSURFACE hSrc, HSURFACE hMask,
				DRect *pSrcRect, int destX, int destY, HDECOLOR hColor);

			// Draws hSrc onto hDest, but fills in the nontransparent pixels with the
			// color you specify.
			void		(*DrawSurfaceSolidColor)(HSURFACE hDest, HSURFACE hSrc,
				DRect *pSrcRect, int destX, int destY, HDECOLOR hTransColor, HDECOLOR hFillColor);
			
			// Draws the source surface onto the dest surface.  You can specify a rectangle 
			// in the source surface and the destination coordinates on the destination
			// surface.. it doesn't scale the bitmap..  If you pass in NULL for the source
			// rect, it'll just use the whole thing.
			void		(*DrawSurfaceToSurface)(HSURFACE hDest, HSURFACE hSrc, 
				DRect *pSrcRect, int destX, int destY);

			void		(*DrawSurfaceToSurfaceTransparent)(HSURFACE hDest, HSURFACE hSrc, 
				DRect *pSrcRect, int destX, int destY, HDECOLOR hColor);

			// Scales the source surface rectangle into the dest rectangle..
			void		(*ScaleSurfaceToSurface)(HSURFACE hDest, HSURFACE hSrc,
				DRect *pDestRect, DRect *pSrcRect);

			void		(*ScaleSurfaceToSurfaceTransparent)(HSURFACE hDest, HSURFACE hSrc,
				DRect *pDestRect, DRect *pSrcRect, HDECOLOR hColor);

			void		(*ScaleSurfaceToSurfaceSolidColor)(HSURFACE hDest, HSURFACE hSrc,
				DRect *pDestRect, DRect *pSrcRect, HDECOLOR hTransColor, HDECOLOR hFillColor);

			// (Affine) warps the source poly into the dest poly.  The source
			// coordinates are CLAMPED to be inside the source surface's rectangle, and
			// the warp is clipped against the destination rectangle (ie: don't specify
			// coordinates outside the source rectangle, but feel free to specify them
			// outside the destination rectangle).  The polygon you specify should be
			// convex.  The minimum number of coordinates is 3 and the maximum
			// is 10.
			void		(*WarpSurfaceToSurface)(HSURFACE hDest, HSURFACE hSrc, 
				DWarpPt *pCoords, int nCoords);

			void		(*WarpSurfaceToSurfaceTransparent)(HSURFACE hDest, HSURFACE hSrc, 
				DWarpPt *pCoords, int nCoords, HDECOLOR hColor);

			void		(*WarpSurfaceToSurfaceSolidColor)(HSURFACE hDest, HSURFACE hSrc, 
				DWarpPt *pCoords, int nCoords, HDECOLOR hTransColor, HDECOLOR hFillColor);

			// Transform the source surface onto the dest surface.  The origin is
			// in the destination surface's coordinates.  If you specify NULL, it will
			// use the centerpoint as the origin.
			void		(*TransformSurfaceToSurface)(HSURFACE hDest, HSURFACE hSrc,
				DFloatPt *pOrigin, int destX, int destY, float angle,
				float scaleX, float scaleY);

			void		(*TransformSurfaceToSurfaceTransparent)(HSURFACE hDest, HSURFACE hSrc,
				DFloatPt *pOrigin, int destX, int destY, float angle,
				float scaleX, float scaleY, HDECOLOR hColor);

			// Draw a filled rectangle into the surface.
			void		(*FillRect)(HSURFACE hDest, DRect *pRect, HDECOLOR hColor);



		// Access to client console variables...

			// Register a console program.  pName is just stored so it should either be
			// static or allocated.  When the client shell DLL is unloaded, it gets 
			// rid of any registered programs.
			// Returns DE_OK or DE_ALREADYEXISTS.
			DRESULT			(*RegisterConsoleProgram)(char *pName, ConsoleProgramFn fn);
			
			// Returns DE_OK or DE_NOTFOUND.
			DRESULT			(*UnregisterConsoleProgram)(char *pName);

			// Returns NULL if the parameter doesn't exist.
			HCONSOLEVAR		(*GetConsoleVar)(char *pName);
			
			// Gets the value of a parameter .. returns 0/NULL if you pass in NULL.
			float			(*GetVarValueFloat)(HCONSOLEVAR hVar);
			char*			(*GetVarValueString)(HCONSOLEVAR hVar);

		// Access to server console mirror variables...

			// Gets the value of a parameter .. returns 0/NULL if you pass in NULL.
			float			(*GetServerConVarValueFloat)(char *pName);
			char*			(*GetServerConVarValueString)(char *pName);


		// Helpers..

			// Register a surface effect.  You should register all your effects in OnEngineInitialized
			// (before you start a world).
			DRESULT		(*AddSurfaceEffect)(SurfaceEffectDesc *pDesc);

			// Turn the input state on or off.  This is for times when the client
			// is interacting with menus and you don't want their mouse movement or
			// keystrokes to get sent to the server.  This defaults to ON.
			void		(*SetInputState)(DBOOL bOn);


			// These access the real, accurate timer.
			float		(*GetTime)();
			float		(*GetFrameTime)();


			// These access the GAME timer (which resides on the server).
			// Since this comes from the server, it will be intermittent, so only 
			// use it for things that must be synced with server.  This timer will
			// not update when there isn't a connection to the server.
			float		(*GetGameTime)();
			float		(*GetGameFrameTime)();

			
			// Print a string to the console.
			void		(*CPrint)(char *pMsg, ...);

			DBOOL		(*IsCommandOn)(int commandNum);

			// Same as typing a string into the console.
			void		(*RunConsoleString)(char *pString);

			// Gives you a pointer to the client shell you created.
			LPCLIENTSHELLDE (*GetClientShell)();

			// Get your client object (NULL if you don't currently have one).
			HLOCALOBJ	(*GetClientObject)();

			// Rotate a rotation around the given vector.
			void		(*RotateAroundAxis)(DRotation *pRotation, DVector *pAxis, float amount);

			// Treat the rotation like Euler angles...
			void		(*EulerRotateX)(DRotation *pRotation, float amount);
			void		(*EulerRotateY)(DRotation *pRotation, float amount);
			void		(*EulerRotateZ)(DRotation *pRotation, float amount);

			// Align a rotation to a normal and spin it.
			// Use pUp to set the frame of reference.
			// If pUp is NULL or the same direction as pVector, then DirectEngine 
			// will kludge an up vector.
			void	(*AlignRotation)(DRotation *pRotation, DVector *pVector, DVector *pUp);

			// Setup the rotation based on euler angles 
			// pitch = rotation around X
			// yaw = rotation around Y
			// roll = rotation around Z
			DRESULT		(*SetupEuler)(DRotation *pRotation, float pitch, float yaw, float roll);

			// Interpolate between two rotations (with quaternions).
			DRESULT		(*InterpolateRotation)(DRotation *pDest, DRotation *pRot1, DRotation *pRot2, float t);

			void		(*GetRotationVectors)(DRotation *pRotation, DVector *pUp, DVector *pRight, DVector *pForward);


			// You can set a global light scale that's applied to all rendering.
			// Colors are RGB 0-1 (values are clamped for you).	
			void		(*GetGlobalLightScale)(DVector *pScale);
			void		(*SetGlobalLightScale)(DVector *pScale);
			void		(*OffsetGlobalLightScale)(DVector *pOffset);

			// Parse a string for arguments.  Works similar to command line parser.
			// args within quotation marks are considered one arg.
			// Returns the all args after pCommand and before end of string or semi-colon.
			//		pCommand - Beginning of string.
			//		pNewCommandPos - Parse fills this in with the ending position
			//		argBuffer - memory Parse can use as scratchpad
			//		argPointers - Parse fills these pointers in to point to args found.
			//		nArgs - Parse fills in the number of arguments found.
			//		Return - DTRUE, semicolon found, more args in string.  DFALSE, no more args.
			int			(*Parse)(char *pCommand, char **pNewCommandPos, char *argBuffer, char **argPointers, int *nArgs);


		// Messaging.

			// Start a message, put data into it (no more than 50 bytes), and
			// end the message..  You can only have one HMESSAGEWRITE open at a time.
			HMESSAGEWRITE	(*StartMessage)(DBYTE messageID);
			DRESULT			(*WriteToMessageFloat)(HMESSAGEWRITE hMessage, float val);
			DRESULT			(*WriteToMessageByte)(HMESSAGEWRITE hMessage, DBYTE val);
			DRESULT			(*WriteToMessageDWord)(HMESSAGEWRITE hMessage, DDWORD val);
			DRESULT			(*WriteToMessageString)(HMESSAGEWRITE hMessage, char *pStr);
			DRESULT			(*WriteToMessageVector)(HMESSAGEWRITE hMessage, DVector *pVal); // 12 bytes
			DRESULT			(*WriteToMessageCompVector)(HMESSAGEWRITE hMessage, DVector *pVal); // 9 bytes
			DRESULT			(*WriteToMessageRotation)(HMESSAGEWRITE hMessage, DRotation *pVal);
			DRESULT			(*WriteToMessageHString)(HMESSAGEWRITE hMessage, HSTRING hString);
			
			// Some notes about this function.  This only works for objects created by the
			// server.  If you put an object in here created by the client, then the receiver
			// will get NULL out.  If the object has been removed on the server before this 
			// message gets read, the receiver will wind up with a different object handle, 
			// so be careful!
			DRESULT			(*WriteToMessageObject)(HMESSAGEWRITE hMessage, HLOCALOBJ hObj);
			
			void			(*EndMessage)(HMESSAGEWRITE hMessage);

			// When your OnMessage function is called, use the handle you're given
			// to read the message data with these functions.
			float			(*ReadFromMessageFloat)(HMESSAGEREAD hMessage);
			DBYTE			(*ReadFromMessageByte)(HMESSAGEREAD hMessage);
			DDWORD			(*ReadFromMessageDWord)(HMESSAGEREAD hMessage);
			char*			(*ReadFromMessageString)(HMESSAGEREAD hMessage);
			void			(*ReadFromMessageVector)(HMESSAGEREAD hMessage, DVector *pVal); // 12 bytes
			void			(*ReadFromMessageCompVector)(HMESSAGEREAD hMessage, DVector *pVal); // 9 bytes
			void			(*ReadFromMessageRotation)(HMESSAGEREAD hMessage, DRotation *pVal);
			HSTRING			(*ReadFromMessageHString)(HMESSAGEREAD hMessage);
			
			// Note: this could very well return NULL if the object sent doesn't still exist.
			// The object will NOT be there if you send a message with the object in it right
			// after creating it.  The server must process at least one update on an object
			// before you can send it to the client.
			HLOCALOBJ		(*ReadFromMessageObject)(HMESSAGEREAD hMessage);


		// Management of client-side objects.

			HLOCALOBJ	(*CreateObject)(ObjectCreateStruct *pStruct);
			DRESULT		(*DeleteObject)(HLOCALOBJ hObj);

			// Gets the objects attached to this object.  outListSize is filled in with how many
			// objects it filled into inList and outNumAttachment is the actual number of attachments
			// (which can be larger than outListSize if inListSize is too small to fit them all).
			DRESULT		(*GetAttachments)(HLOCALOBJ hObj, HLOCALOBJ *inList, DDWORD inListSize, 
				DDWORD *outListSize, DDWORD *outNumAttachments);

			// Change position and rotation.  It's more efficient to set them 
			// at the same time...
			void		(*GetObjectPos)(HLOCALOBJ hObj, DVector *pPos);
			void		(*SetObjectPos)(HLOCALOBJ hObj, DVector *pPos);

			void		(*GetObjectRotation)(HLOCALOBJ hObj, DRotation *pRotation);
			void		(*SetObjectRotation)(HLOCALOBJ hObj, DRotation *pRotation);
			void		(*SetObjectPosAndRotation)(HLOCALOBJ hObj, DVector *pPos, DRotation *pRotation);

			DDWORD		(*GetObjectType)(HLOCALOBJ hObj);
			
			// Get/Set scale.
			DRESULT		(*GetObjectScale)(HLOCALOBJ hObj, DVector *pScale);
			DRESULT		(*SetObjectScale)(HLOCALOBJ hObj, DVector *pScale);

			// RGB 0-1.
			void		(*GetObjectColor)(HLOCALOBJ hObject, float *r, float *g, float *b, float *a);
			void		(*SetObjectColor)(HLOCALOBJ hObject, float r, float g, float b, float a);

			DDWORD		(*GetObjectFlags)(HLOCALOBJ hObj);
			void		(*SetObjectFlags)(HLOCALOBJ hObj, DDWORD flags);

			// Get/Set object user flags.  Can't set user flags on an object
			// created on the server.
			DRESULT		(*GetObjectUserFlags)(HLOCALOBJ hObj, DDWORD *pFlags);
			DRESULT		(*SetObjectUserFlags)(HLOCALOBJ hObj, DDWORD flags);
			
			// Get/set the client flags (defined above).
			DDWORD		(*GetObjectClientFlags)(HLOCALOBJ hObj);
			void		(*SetObjectClientFlags)(HLOCALOBJ hObj, DDWORD flags);

			// User data for the object..
			void*		(*GetObjectUserData)(HLOCALOBJ hObj);
			void		(*SetObjectUserData)(HLOCALOBJ hObj, void *pData);


		// Camera functions.

			// Get/Set a camera's FOV.  It defaults to (PI/2, PI/2).
			// It will clamp your values between (PI/100, 99*PI/100)
			void		(*GetCameraFOV)(HLOCALOBJ hObj, float *pX, float *pY);
			void		(*SetCameraFOV)(HLOCALOBJ hObj, float fovX, float fovY);

			// Get/Set the camera Z order.  Defaults to 0.  Cameras with
			// higher Z orders are rendered after cameras with lower Z orders.
			// Cameras with Z order of -1 won't be rendered.
			int			(*GetCameraZOrder)(HLOCALOBJ hObj);
			void		(*SetCameraZOrder)(HLOCALOBJ hObj, int zOrder);
			
			void		(*GetCameraRect)(HLOCALOBJ hObj, DBOOL *bFullscreen,
				int *left, int *top, int *right, int *bottom);
			
			// Set the camera's rectangle on the screen.
			// If bFullscreen is DTRUE, then it ignores the rect and draws the
			// camera fullscreen.  If the rectangle extends over the screen
			// boundaries, then it is clipped..
			void		(*SetCameraRect)(HLOCALOBJ hObj, DBOOL bFullscreen, 
				int left, int top, int right, int bottom);

			// Get/Set the camera light add.  RGB 0-1.  Light add is applied AFTER
			// scaling, so if light is fully bright and scaling is zero, you'll just
			// see whiteness.  When the light add is nonzero, it draws a poly over 
			// the screen so don't use it all the time!
			// These return DFALSE if the object is not a camera.
			DBOOL		(*GetCameraLightAdd)(HLOCALOBJ hCamera, DVector *pAdd);
			DBOOL		(*SetCameraLightAdd)(HLOCALOBJ hCamera, DVector *pAdd);

			// Renders the list of objects through the given camera.
			DBOOL		(*RenderObjects)(HLOCALOBJ hCamera, HLOCALOBJ *pObjects, int nObjects);


		// Particle system manipulation.

			// gravityAccel default is -500
			// flags default is 0
			// particleRadius default is 300
			// color scale defauls to 1.0
			// Particle colors are 0-255.
			
			// All particle positions are RELATIVE to the particle system's 
			// position and rotation.  In many cases, you can have your code be very simple
			// and fast if you just move and rotate the particle system and not the particles.
			
			// Change the system's parameters.
			void		(*SetupParticleSystem)(HLOCALOBJ hObj, char *pTextureName, float gravityAccel, DDWORD flags, float particleRadius);

			// Set a color scale for all the particles in the system.  Clamped to (0.0, 1.0), defaults to 1.0.
			float		(*GetParticleSystemColorScale)(HLOCALOBJ hObj);
			void		(*SetParticleSystemColorScale)(HLOCALOBJ hObj, float scale);
			
			void		(*AddParticle)(HLOCALOBJ hObj, DVector *pPos, DVector *pVelocity, DVector *pColor, float lifeTime);
			void		(*AddParticles)(HLOCALOBJ hObj, DDWORD nParticles,
				DVector *pMinOffset, DVector *pMaxOffset,
				DVector *pMinVelocity, DVector *pMaxVelocity, 
				DVector *pMinColor, DVector *pMaxColor,
				float minLifetime, float maxLifetime);

			DBOOL		(*GetParticles)(HLOCALOBJ hObj, DEParticle **pHead, DEParticle **pTail);

			// Get/Set particle positions.  hSystem is NOT checked to be valid here
			// for speed so make sure it's valid!
			void		(*GetParticlePos)(HLOCALOBJ hSystem, DEParticle *pParticle, DVector *pPos);
			void		(*SetParticlePos)(HLOCALOBJ hSystem, DEParticle *pParticle, DVector *pPos);

			// Remove a particle.
			void		(*RemoveParticle)(HLOCALOBJ hSystem, DEParticle *pParticle);

			// This is an optimization you can make to help the engine minimize its boundaries on
			// a particle system.  If you create particles in various places and they go away, you
			// can use this every so often to recalculate where the particles are.
			DRESULT		(*OptimizeParticles)(HLOCALOBJ hSystem);
			


		// Line system manipulation.
		// As with particle systems, the lines are centered around the object's origin.
		// Don't just place the object at the origin and put lines way off to the side,
		// it's more efficient to keep the lines as close to the center as possible.

			// Set hPrev to NULL to start, then pass in the return value, etc..
			// Returns DNULL for last line.
			// ** If you call RemoveLine on the current HDELINE, DO NOT
			// ** pass that into GetNextLine - call GetNextLine first
			// ** while the HDELINE is still valid!
			HDELINE		(*GetNextLine)(HLOCALOBJ hObj, HDELINE hPrev);

			void		(*GetLineInfo)(HDELINE hLine, DELine *pLine);
			void		(*SetLineInfo)(HDELINE hLine, DELine *pLine);

			// Adds a line to the end of the line system's list.
			HDELINE		(*AddLine)(HLOCALOBJ hObj, DELine *pLine);
			void		(*RemoveLine)(HLOCALOBJ hObj, HDELINE hLine);


		// Poly grid manipulation.

			// A poly grid is basically a heightmapped grid of pixels that are drawn
			// as polygons.  Each pixel can have a value from -127 to 127.  The pixel's value 
			// defines its height and is a lookup into the color table for the vertex color.
			// You can scale and rotate the poly grid using SetObjectScale and SetObjectRotation.
			// bHalfTriangles will cause it to look VERY triangulated, but draw way faster.
			DBOOL		(*SetupPolyGrid)(HLOCALOBJ hObj, DDWORD width, DDWORD height, DBOOL bHalfTrianges);

			// Set the texture.  The texture MUST be a sprite file.  It CANNOT be a .dtx file!
			DRESULT		(*SetPolyGridTexture)(HLOCALOBJ hObj, char *pName);

			// Get/Set the texture pan and scale for a PolyGrid.
			// Defaults are 0.0 for xPan and yPan, and 1.0 for xScale and yScale.
			DRESULT		(*GetPolyGridTextureInfo)(HLOCALOBJ hObj, float *xPan, float *yPan, float *xScale, float *yScale);
			DRESULT		(*SetPolyGridTextureInfo)(HLOCALOBJ hObj, float xPan, float yPan, float xScale, float yScale);
			
			// You can set it to be transparent or not.  It defaults to not being transparent.
			DRESULT		(*SetPolyGridTransparency)(HLOCALOBJ hObj, DBOOL bTransparent);
			DRESULT		(*GetPolyGridInfo)(HLOCALOBJ hObj, char **pBytes, DDWORD *pWidth, DDWORD *pHeight, DVector **pColorTable);

			// Set pMin and pMax to the dimensions of the box you want the polygrid to fit in.
			// pPos and pScale will be filled in with the recommended position and scale.
			DRESULT		(*FitPolyGrid)(HLOCALOBJ hObj, DVector *pMin, DVector *pMax, DVector *pPos, DVector *pScale);


		// Light manipulation.

			// Get/Set a light's color (RGB, 0.0f to 1.0f).
			// When you create a light, its color defaults to (0,0,0).
			void	(*GetLightColor)(HLOCALOBJ hObj, float *r, float *g, float *b);
			void	(*SetLightColor)(HLOCALOBJ hObj, float r, float g, float b);

			// Get/Set a light's radius.
			// When you create a light, its radius defaults to 100.
			float	(*GetLightRadius)(HLOCALOBJ hObj);
			void	(*SetLightRadius)(HLOCALOBJ hObj, float radius);


		// Container manipulation.

			// Get a container's code.
			// Returns DFALSE if the object is not a container.
			DBOOL	(*GetContainerCode)(HLOCALOBJ hObj, D_WORD *pCode);

			// Find out what containers contain the given point.
			// Returns the number of containers filled in.
			DDWORD	(*GetPointContainers)(DVector *pPoint, HLOCALOBJ *pList, DDWORD maxListSize);


		// Sprite manipulation.

			// This clips the sprite on the poly.
			// Returns DE_OK or DE_ERROR if not a sprite.
			// Pass in INVALID_HPOLY to un-clip the sprite.
			DRESULT	(*ClipSprite)(HLOCALOBJ hObj, HPOLY hPoly);


		// Client-side models..
		
			// Iterate through the model's nodes.  Returns DE_FINISHED when there are no more.
			// hCurNode = INVALID_MODEL_NODE;
			// while(interface->GetNextModelNode(hModel, hCurNode, &hCurNode) == DE_OK)
			// { ... }
			DRESULT	(*GetNextModelNode)(HLOCALOBJ hObject, HMODELNODE hNode, HMODELNODE *pNext);

			// Get a model node's name.
			DRESULT	(*GetModelNodeName)(HLOCALOBJ hObject, HMODELNODE hNode, char *pName, DDWORD maxLen);

			// Hide/unhide a node on the model (they're all unhidden by default).
			// Returns DE_OK, DE_ERROR, LT_NOCHANGE, or DE_NODENOTFOUND.
			DRESULT	(*GetModelNodeHideStatus)(HOBJECT hObj, char *pNodeName, /* out */DBOOL *bHidden);
			DRESULT	(*SetModelNodeHideStatus)(HOBJECT hObj, char *pNodeName, DBOOL bHidden);

			// Returns the animation the model is currently on.  (DDWORD)-1 if none.
			DDWORD	(*GetModelAnimation)(HLOCALOBJ hObj);
			void	(*SetModelAnimation)(HLOCALOBJ hObj, DDWORD iAnim);

			// Starts the current animation over.
			DRESULT	(*ResetModelAnimation)(HLOCALOBJ hObj);

			// Tells what the playback state of the model is (a combination of the
			// MS_ bits defined in basedefs_de.h).
			DDWORD	(*GetModelPlaybackState)(HLOCALOBJ hObj);

			// Get/Set the looping state of the model.  The default state is TRUE.
			DBOOL	(*GetModelLooping)(HLOCALOBJ hObj);
			void	(*SetModelLooping)(HLOCALOBJ hObj, DBOOL bLoop);


		// (Geometry) surface functions.

			DBOOL	(*GetSurfaceBounds)(SurfaceData *pSurface, DVector *pMin, DVector *pMax);

			// Hide/Unhide a poly.
			DRESULT	(*SetPolyHideStatus)(HPOLY hPoly, DBOOL bHide);

			// Just used for internal debugging.. never use this.
			DRESULT	(*GetPolyIndex)(HPOLY hPoly, DDWORD *pIndex);

			// Get the texture flags from a poly.  Returns DE_OK
			// or DE_ERROR if no world is loaded or hPoly is invalid.
			DRESULT	(*GetPolyTextureFlags)(HPOLY hPoly, DDWORD *pFlags);

		
		// Render hooks.

			// When this is set, the renderer will call your hook function before drawing each
			// model.  Either keep the function very fast or set it to NULL so you don't slow
			// the renderer down.
			DRESULT	(*SetModelHook)(ModelHookFn fn, void *pUser);

	} ClientDE;


#endif  // __CLIENT_DE_H__

