
// This module defines all the DirectEngine return codes.

#ifndef __DE_RETURNCODES_H__
#define __DE_RETURNCODES_H__

	// DRESULT is used for all return values.
	// High 8 bits: error flags
	// Low 16 bits: error code
	#define ERROR_FLAGS(err) ((err) & 0xFF000000)
	#define ERROR_CODE(err) ((err) & 0xFFFF)

	typedef unsigned long DRESULT;


	// --------------------------------------------------------------- //
	// Flags.
	// --------------------------------------------------------------- //
	
	// Portal flags.
	#define PORTAL_OPEN		1

	// Video playback flags.
	#define PLAYBACK_FULLSCREEN	1
	#define PLAYBACK_YINTERLACE	2

	// Intersect flags.
	#define INTERSECT_OBJECTS		1	// Try to hit objects.. small speed hit by doing so.
	#define IGNORE_NONSOLID			2	// Ignore nonsolid objects and nonsolid surfaces. Just a helper filter.
	#define INTERSECT_HPOLY			4	// Fill in the HPOLY for this intersection (this slows it down a little).


	// --------------------------------------------------------------- //
	// DE_ERROR flags.
	// Note: on an allocation failure, the engine shuts down on the
	//       spot so there is no DE_OUTOFMEMORY error.
	// --------------------------------------------------------------- //
	
	#define ERROR_DISCONNECT	(1<<25)	// Disconnect from the server.
	#define ERROR_SHUTDOWN		(1<<26)	// Shutdown everything.


	// --------------------------------------------------------------- //
	// Return codes.
	// --------------------------------------------------------------- //

	#define LT_OK		0	// No problem!
	#define DE_OK		0	
	
	#define LT_ERROR	1	// Uh oh..
	#define DE_ERROR	1	
	
	#define DE_FINISHED	2	// Done with operation.

	// Video status.
	#define VIDEO_STARTED		10
	#define VIDEO_ERROR			11
	#define VIDEO_PLAYING		12
	#define VIDEO_NOTPLAYING	13

	#define DE_TRIEDTOREMOVECLIENTOBJECT	20	// Tried to remove a client's object.
	#define DE_NOTINWORLD					21	// Tried to do an operation but a world wasn't running.
	#define DE_MISSINGFILE					22	// Missing the requested file.
	#define DE_MISSINGWORLDMODEL			23	// Missing the requested WorldModel.
	#define DE_INVALIDMODELFILE				24	// Invalid model file.
	#define DE_OBJECTNOTINWORLD				25	// Tried to modify an object but it was removed from the world.
	#define DE_CANTREMOVESERVEROBJECT		26	// Can't remove a server object from the client.
	
	#define DE_CANTLOADGAMERESOURCES		27	// Was missing game rezfile or directory.
	#define DE_CANTINITIALIZEINPUT			28	// Unable to initialize input.
	#define DE_MISSINGSHELLDLL				29
	#define DE_INVALIDSHELLDLL				30
	#define DE_INVALIDSHELLDLLVERSION		31
	#define DE_CANTCREATECLIENTSHELL		32
	#define DE_MISSINGSOUNDDLL				33
	#define DE_INVALIDSOUNDDLL				34
	#define DE_UNABLETOINITSOUNDDLL			35
	#define DE_MISSINGMUSICDLL				36
	#define DE_INVALIDMUSICDLL				37
	#define DE_UNABLETOINITMUSICDLL			38
	#define DE_CANTINITDIRECTPLAY			39
	#define DE_USERCANCELED					40	// User canceled connect dialog.
	#define DE_MISSINGWORLDFILE				41	// Missing world file from server.
	#define DE_INVALIDWORLDFILE				42	// Invalid world file.
	#define DE_ERRORBINDINGWORLD			43	// Error binding world to renderer.
	#define DE_INVALIDSERVERPACKET			44	// Got a bad packet from the server.
	#define DE_MISSINGSPRITEFILE			45
	#define DE_INVALIDSPRITEFILE			46
	#define DE_MISSINGMODELFILE				47
	#define DE_UNABLETORESTOREVIDEO			48	// Couldn't restore video mode.
	#define DE_SERVERERROR					49	// Got an error from the server.
	#define DE_CANTCREATESERVER				50	// Was unable to create a server.
	#define DE_ERRORLOADINGRENDERDLL		51	// Error loading the render DLL.

	#define DE_MISSINGCLASS					52	// Missing a needed class from object.dll.
	#define DE_CANTCREATESERVERSHELL		53	// Unable to create a server shell.
	#define DE_INVALIDOBJECTDLL				54	// Invalid (or missing) object DLL.
	#define DE_INVALIDOBJECTDLLVERSION		55	// Invalid object DLL version.
	#define DE_ERRORINITTINGNETDRIVER		56	// Couldn't initialize net driver.
	#define DE_NOGAMERESOURCES				57	// No game resources specified.
	#define DE_CANTRESTOREOBJECT			58	// Couldn't restore an object.

	#define DE_NODENOTFOUND					59	// Couldn't find the specified model node.
	#define DE_INVALIDPARAMS				60	// Invalid parameters passed to function (like NULL for an HOBJECT).

	#define DE_NOTFOUND						61	// Something was not found.
	#define DE_ALREADYEXISTS				62	// Something already exists.
	#define DE_NOTCONNECTED					63	// Not currently on a server.
	
	#define DE_INSIDE						64	// Inside.
	#define DE_OUTSIDE						65	// Outside.

	#define DE_INVALIDDATA					66	// Invalid data.
	#define DE_OUTOFMEMORY					67	// Couldn't get enough memory.  The engine
												// will always shutdown before giving this
												// error, so it's only used in tools.
	#define DE_MISSINGPALETTE				68	// Internal.
	#define DE_INVALIDVERSION				69	// Invalid version.

	#define LT_NOCHANGE						70	// Nothing was changed.
		

#endif  // __DE_RETURNCODES_H__




