
#ifndef __SERVERSHELL_DE_H__
#define __SERVERSHELL_DE_H__


	#include "basedefs_de.h"
	#include "engineobjects_de.h"
	#include "server_de.h"



	/////////////////////////////////////////////////////////////////////
	// ServerShellDE interface.  You must implement this for a server.
	/////////////////////////////////////////////////////////////////////

	typedef struct ServerShellDE_t
	{

		// This is so DirectEngine will skip over a C++ object's VTable.
		#ifdef COMPILE_WITH_C
			void		*cpp_4BytesForVTable;
		#endif
		
		// Notification when new clients log into the server.
		void	(*OnAddClient)(struct ServerShellDE_t *pShell, HCLIENT hClient);
		void	(*OnRemoveClient)(struct ServerShellDE_t *pShell, HCLIENT hClient);

		// You must create an object to represent the client.
		// It uses the object's position to determine what the client can see.
		LPBASECLASS	(*OnClientEnterWorld)(struct ServerShellDE_t *pShell, HCLIENT hClient);
		void		(*OnClientExitWorld)(struct ServerShellDE_t *pShell, HCLIENT hClient);

		// Called before and after you switch worlds.
		void	(*PreStartWorld)(struct ServerShellDE_t *pShell, DBOOL bSwitchingWorlds);
		void	(*PostStartWorld)(struct ServerShellDE_t *pShell);		

		// Message handler.
		void	(*OnMessage)(struct ServerShellDE_t *pShell, HCLIENT hSender, DBYTE messageID, HMESSAGEREAD hMessage);

		// Handler for messages from objects.
		void	(*OnObjectMessage)(struct ServerShellDE_t *pShell, LPBASECLASS pSender, DDWORD messageID, HMESSAGEREAD hMessage);

		// Command notification.
		void	(*OnCommandOn)(struct ServerShellDE_t *pShell, HCLIENT hClient, int command);
		void	(*OnCommandOff)(struct ServerShellDE_t *pShell, HCLIENT hClient, int command);

		// Update loop callback.. do whatever you like in here.
		// Time since the last Update() call is passed in.
		void	(*Update)(struct ServerShellDE_t *pShell, DFLOAT timeElapsed);

		// This is set after you create and return your ServerShell.
		struct ServerDE_t *m_pServerDE;

	} ServerShellDE;



	#define SERVERSHELL_VERSION 1

	typedef ServerShellDE* (*CreateServerShellFn)();
	typedef void (*DeleteServerShellFn)(ServerShellDE *);
	
	// You must implement these two functions to create and delete your server shell.
	ServerShellDE* CreateServerShell();
	void DeleteServerShell(ServerShellDE *);


	#define SETUP_SERVERSHELL()\
		BEGIN_EXTERNC() \
			__declspec(dllexport) void GetServerShellFunctions(CreateServerShellFn *pCreate, DeleteServerShellFn *pDelete);\
			__declspec(dllexport) int GetServerShellVersion();\
		END_EXTERNC()\
		void GetServerShellFunctions(CreateServerShellFn *pCreate, DeleteServerShellFn *pDelete)\
		{\
			*pCreate = CreateServerShell;\
			*pDelete = DeleteServerShell;\
		}\
		int GetServerShellVersion()\
		{\
			return SERVERSHELL_VERSION;\
		}


#endif  // __SERVERSHELL_DE_H__
