
// --------------------------------
// File    : EffectManager.cpp
// Function: Handles the various special effects; adding, removing and updating them
//         : as appropriate.
// --------------------------------

#include "de_client.h"
#include "effectmanager.h"
#include "demo_msgtypes.h"

#include "windows.h"
#include "winbase.h"

// Include Effect files here
#include "lensflare.h"
#include "polygrid.h"
#include "waterspray.h"
#include "torch.h"

// ----------------
// Handler
//		Receives SpecialEffects messages and dispatches them to the correct effects.
// ----------------
void EffectManager::Handler(HLOCALOBJ hObj, HMESSAGEREAD hMsg)
{
	if (!m_pPlayer) return;

	DBYTE byType;

	// Read the message type from our message.
	byType = m_pPlayer->GetClient()->ReadFromMessageByte(hMsg);

	// Which type is it...
	switch(byType)
	{
		case DEMO_MSG_LENSFLARE_ADD:
			// It's a lensflare
			LensFlare *pFlare;

			pFlare = new LensFlare();
			if (pFlare->Init(m_pPlayer,hObj,byType,hMsg))
				AddEffect(pFlare);
			else
				delete pFlare;

			break;
		case DEMO_MSG_POLYGRID_ADD:
			// It's a polygrid
			PolyGrid *pGrid;

			pGrid = new PolyGrid();
			if (pGrid->Init(m_pPlayer,hObj,byType,hMsg))
				AddEffect(pGrid);
			else
				delete pGrid;

			break;
		case DEMO_MSG_WATERSPRAY_ADD:
			// It's a waterspray
			WaterSpray *pSpray;

			pSpray = new WaterSpray();
			if (pSpray->Init(m_pPlayer,hObj,byType,hMsg))
				AddEffect(pSpray);
			else
				delete pSpray;

			break;
		case DEMO_MSG_TORCH_ADD:
			// It's a torch
			Torch *pTorch;

			pTorch = new Torch();
			if (pTorch->Init(m_pPlayer,hObj,byType,hMsg))
				AddEffect(pTorch);
			else
				delete pTorch;

			break;

		default:
			// We don't know what it is.
			m_pPlayer->GetClient()->CPrint("EffectManager: unknown effect %d", byType);
			break;
	}
}

// ----------------
// Update
//		Updates all active effects
// ----------------
void EffectManager::Update()
{
	Effect *pEffect, *pTemp;

	// Get the head of our effect list.
	pEffect = m_pEffects;

	// Walk the list
	while (pEffect)
	{
		pTemp = pEffect->GetNext();

		// Perform the update
		if (!pEffect->Update())
			DelEffect(pEffect);

		pEffect = pTemp;
	}
}

// ----------------
// AddEffect
//		Adds an Effect to the Effect list.
// ----------------
void EffectManager::AddEffect(Effect *pEffect)
{
	pEffect->SetNext(m_pEffects);
	pEffect->SetPrev(NULL);

	if (m_pEffects)
		m_pEffects->SetPrev(pEffect);

	m_pEffects = pEffect;
}

// ----------------
// DelEffect
//		Deletes an Effect from the Effect list.
// ----------------
void EffectManager::DelEffect(Effect *pEffect)
{
	// Make sure our list is preserved intact
	if (pEffect->GetNext())
		pEffect->GetNext()->SetPrev(pEffect->GetPrev());

	if (pEffect->GetPrev())
		pEffect->GetPrev()->SetNext(pEffect->GetNext());
	else
		m_pEffects = pEffect->GetNext();

	// Allow the effect to clean up...
	pEffect->Destroy();

	// And delete the pointer
	delete pEffect;
}

// ----------------
// DestroyAll
//		Destroys the entire Effects list.
// ----------------
void EffectManager::DestroyAll()
{
	Effect *pEffect, *pTemp;

	pEffect = m_pEffects;

	while (pEffect)
	{
		pTemp = pEffect->GetNext();

		pEffect->Destroy();
		delete pEffect;

		pEffect = pTemp;
	}
}