
// --------------------------------
// File    : LensFlare.cpp
// Function: C++ class wrapper around the C functions in c_lensflare.c
//         : Allows those to be used in the EffectManager.
// --------------------------------


#include "de_client.h"
#include "LensFlare.h"

LensFlare::LensFlare() : Effect()
{
	m_pPlayer = NULL;
	m_hFlare = NULL;
}

// ----------------
// Init
//		Initializes the LensFlare
// ----------------
DBOOL LensFlare::Init(ClientPlayer *pPlayer, HLOCALOBJ hObj, DBYTE byType, HMESSAGEREAD hMsg)
{
	m_pPlayer = pPlayer;
	m_hServerObject = hObj;
	
	CreateFlare(hObj);

	return DTRUE;
}

// ----------------
// CreateFlare
//		Actually creates the flare.
// ----------------
void LensFlare::CreateFlare(HLOCALOBJ hObj)
{
	// Do we have a client?
	if (m_pPlayer)
	{
		// Do we have a flare already?
		if (m_hFlare)
		{
			// Nuke it, since we're going to replace it.
			DestroyFlare();
		}

		// Make that nifty lensflare.
		m_hFlare = lf_CreateFlare(m_pPlayer->GetClient(),hObj);
		m_hServerObject = hObj;
	}
}

// ----------------
// DestroyFlare
//		Actually destroys the flare.
// ----------------
void LensFlare::DestroyFlare()
{
	// Do we have a client?
	if (m_pPlayer)
	{
		// Do we have a flare?
		if (m_hFlare)
		{
			// Nuke it.
			lf_DestroyFlare(m_pPlayer->GetClient(),m_hFlare);
		}
	}
}

// ----------------
// Update
//		Updates the lensflare.
// ----------------
DBOOL LensFlare::Update()
{
	// Do we have a client?
	if (m_pPlayer)
	{
		// Do we have a flare?
		if (m_hFlare)
		{
			ClientIntersectQuery iQuery;
			ClientIntersectInfo iInfo;

			m_pPlayer->GetClient()->GetObjectPos(m_pPlayer->GetCamera(), &iQuery.m_To);
			m_pPlayer->GetClient()->GetObjectPos(m_hServerObject, &iQuery.m_From);

			// If we can see the light source...
			if (!m_pPlayer->GetClient()->IntersectSegment(&iQuery,&iInfo))
			{
				// ...and if we're close enough...
				if (VEC_DIST(iQuery.m_To,iQuery.m_From) <= 1000.0f)
				{
					// Call into the lower-level C functions and update it.
					lf_UpdateFlare(m_pPlayer->GetClient(), m_hFlare, m_pPlayer->GetCamera());
				}
			} else
			{
				// Otherwise, hide it.
				lf_HideFlare(m_pPlayer->GetClient(), m_hFlare);
			}
		}
	}

	return DTRUE;
}

