
// --------------------------------
// File    : LokiCam.h
// Function: Cheap hack camera class, provided as an example of various types
//         : of camera modes.  All are intended more as starting points for
//         : user extensions.  Partly inspired by Ashley Matheson's ChaseCam
//         : tutorial.
// --------------------------------


#ifndef __LOKICAM_H
#define __LOKICAM_H

#define LOKICAM_MODE_RELATIVE		0
#define LOKICAM_MODE_ISO			1
#define LOKICAM_MODE_GAUNTLET		2
#define LOKICAM_MODE_GAUNTLET2		3
#define LOKICAM_MODE_FREEFORM		4

#define LOKICAM_MODE_LASTMODE		LOKICAM_MODE_FREEFORM

class LokiCam {
protected:

	DBYTE		m_CameraMode;
	DRotation   m_CameraRotation;
	DVector		m_CameraPos;
	float		m_CameraZoom;
	float		m_Yaw, m_Roll, m_Pitch;
	float		m_Offset;
	HLOCALOBJ	m_hTarget;
	HLOCALOBJ	m_hCamera;
	DBOOL		m_bWasBlocked;
	float		m_fSteps, m_fCurrentStep;
	ClientDE	*m_pClientDE;
	
public:
	LokiCam();
	~LokiCam();

	void					Init(ClientDE *pClient);

	HLOCALOBJ				GetCamera() { return m_hCamera; };
	DRotation				GetRotation() { return m_CameraRotation; };

	void					SetTarget(HLOCALOBJ hObj) { m_hTarget = hObj; };
	void					SetCamera(HLOCALOBJ hCam) { m_hCamera = hCam; };

	void					CycleMode();
	void					Zoom(float delta);
	
	void					Update();

	void					ResetCamera();

	// Commands for free-flow camera
	void					UpdateOffset(float delta);
	void					UpdateYaw(float delta);
	void					UpdateRoll(float delta);
	void					UpdatePitch(float delta);

private:
	void					UpdateRelative();
	void					UpdateIso();
	void					UpdateGauntlet();
	void					UpdateGauntlet2();
	void					UpdateFreeform();
};

#endif // __LOKICAM_H