
// --------------------------------
// File    : PolyGrid.cpp
// Function: PolyGrid class for the PolyGrid (water) effect
//         : Special thanks to Kevin Stephens (of the Riot: Mobile Armor team)
//         : for the use of his polygrid class as the basis for this one.
// --------------------------------


#include "de_client.h"
#include "polygrid.h"
#include "windows.h"
#include "winbase.h"

static DFLOAT s_PolyGridYaw		= 0.0f;
static DFLOAT s_PolyGridPitch	= 0.0f;
static DFLOAT s_PolyGridRoll	= 0.0f;

static char g_SinTable[256];
static DBOOL g_bSinTableInitted=DFALSE;

PolyGrid::PolyGrid() : Effect() 
{
	VEC_INIT(m_vDims);
	VEC_SET(m_vColor1, 255.0f, 255.0f, 255.0f);
	VEC_SET(m_vColor2, 255.0f, 255.0f, 255.0f);
	m_fXScaleMin		= 15.0f; 
	m_fXScaleMax		= 25.0f; 
	m_fYScaleMin		= 15.0f; 
	m_fYScaleMax		= 25.0f; 
	m_fXScaleDuration	= 10.0f;
	m_fYScaleDuration	= 10.0f;
	m_fXPan				= 0.0f;
	m_fYPan				= 0.0f;
	m_hstrSurfaceSprite	= DNULL;
	m_bTransparent		= DTRUE;

	m_DistanceGrid		= DNULL;
	m_fCount			= 0.0f;

	m_bScalingXUp = DTRUE;
	m_bScalingYUp = DTRUE;

	m_dwNumPolies = 160;
}

// ----------------
// Init
//		Initialize the Polygrid.
// ----------------
DBOOL PolyGrid::Init(ClientPlayer *pPlayer, HLOCALOBJ hObj, DBYTE byType, HMESSAGEREAD hMessage)
{
	m_pPlayer = pPlayer;
	m_hServerObject = hObj;
	char *temp;

	// Read our configuration from the message
	m_pPlayer->GetClient()->ReadFromMessageVector(hMessage,&m_vDims);
	m_pPlayer->GetClient()->ReadFromMessageVector(hMessage,&m_vColor1);
	m_pPlayer->GetClient()->ReadFromMessageVector(hMessage,&m_vColor2);
	m_fXScaleMin = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	m_fXScaleMax = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	m_fYScaleMin = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	m_fYScaleMax = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	m_fXScaleDuration = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	m_fYScaleDuration = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	m_fXPan = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	m_fYPan = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	m_bTransparent = (m_pPlayer->GetClient()->ReadFromMessageByte(hMessage) != 0);
	m_dwNumPolies = m_pPlayer->GetClient()->ReadFromMessageDWord(hMessage);

	temp = m_pPlayer->GetClient()->ReadFromMessageString(hMessage);

	m_hstrSurfaceSprite = m_pPlayer->GetClient()->CreateString(temp);

	HSTRING hstrTemp;

	hstrTemp = m_pPlayer->GetClient()->CreateString("none");

	if (m_pPlayer->GetClient()->CompareStrings(m_hstrSurfaceSprite,hstrTemp))
	{
		m_pPlayer->GetClient()->FreeString(m_hstrSurfaceSprite);
		m_hstrSurfaceSprite = DNULL;
	}

	m_pPlayer->GetClient()->FreeString(hstrTemp);

	return CreateObject(m_pPlayer->GetClient());
}

// ----------------
// Destroy
//		Destroys the PolyGrid.
// ----------------
void PolyGrid::Destroy()
{
	if (m_DistanceGrid)
	{
		free(m_DistanceGrid);
		m_DistanceGrid = DNULL;
	}

	if (m_hstrSurfaceSprite && m_pPlayer->GetClient())
	{
		m_pPlayer->GetClient()->FreeString(m_hstrSurfaceSprite);
	}
}

// ----------------
// CreateObject
//		Create the actual PolyGrid object.
// ----------------
DBOOL PolyGrid::CreateObject(ClientDE *pClientDE)
{
	// Build a color ramp.
	ColorRamp colorRamps[] = 
	{
		m_vColor1.x, m_vColor1.y, m_vColor1.z, 0,
		m_vColor2.x, m_vColor2.y, m_vColor2.z, 256
	};

	// Sanity check!
	if (!pClientDE) return DFALSE;

	DVector vPos;
	pClientDE->GetObjectPos(m_hServerObject, &vPos);

	DRotation rRot;
	pClientDE->GetObjectRotation(m_hServerObject, &rRot);

	// Setup the PolyGrid...

	ObjectCreateStruct createStruct;
	INIT_OBJECTCREATESTRUCT(createStruct);

	createStruct.m_ObjectType = OT_POLYGRID;
	createStruct.m_Flags = FLAG_VISIBLE;
	VEC_COPY(createStruct.m_Pos, vPos);
	ROT_COPY(createStruct.m_Rotation, rRot);

	DDWORD dwSize = 2;
	if (m_dwNumPolies > 4)
	{
 		dwSize = (DDWORD)sqrt((DFLOAT)(m_dwNumPolies/2)) + 1;
	}

	m_hObject = pClientDE->CreateObject(&createStruct);
	pClientDE->SetupPolyGrid(m_hObject, dwSize, dwSize, DFALSE);
	pClientDE->SetPolyGridTransparency(m_hObject, m_bTransparent);
	
	DVector vMin, vMax, vScale;
	VEC_SUB(vMin, vPos, m_vDims);
	VEC_ADD(vMax, vPos, m_vDims);

	pClientDE->FitPolyGrid(m_hObject, &vMin, &vMax, &vPos, &vScale);

	// If we have a surface sprite, set it up.
	if (m_hstrSurfaceSprite)
	{
		char* pSpriteName = m_pPlayer->GetClient()->GetStringData(m_hstrSurfaceSprite);
		if (pSpriteName && pSpriteName[0])
		{
			m_pPlayer->GetClient()->SetPolyGridTexture(m_hObject, pSpriteName);
			m_pPlayer->GetClient()->SetPolyGridTextureInfo(m_hObject, m_fXPan, m_fYPan, m_fXScaleMin, m_fYScaleMin);
		}
	}

	m_pPlayer->GetClient()->SetObjectPos(m_hObject, &vPos);
	m_pPlayer->GetClient()->SetObjectScale(m_hObject, &vScale);

	SetPolyGridPalette(colorRamps, sizeof(colorRamps)/sizeof(colorRamps[0]));
	PrecalculatePlasma();

	return DTRUE;
}

// ----------------
// Update
//		Updates the Polygrid effect
// ----------------
DBOOL PolyGrid::Update()
{
	if(!m_hObject || !m_pPlayer->GetClient()) return DFALSE;

	// Don't update if not drawn :)
	if (!(m_pPlayer->GetClient()->GetObjectFlags(m_hObject) & FLAG_WASDRAWN)) 
	{
		return DTRUE;
	}


	UpdatePlasma();

	if (m_hstrSurfaceSprite)
	{
		UpdateSurface();
	}


	// Update the position of the polygrid to reflect the position of the
	// server object...

	DVector vPos;
	m_pPlayer->GetClient()->GetObjectPos(m_hServerObject, &vPos);
	m_pPlayer->GetClient()->SetObjectPos(m_hObject, &vPos);

	return DTRUE;
}


// ----------------
// UpdateSurface
//		Updates the surface of the polygrid.
// ----------------
void PolyGrid::UpdateSurface()
{
	if (!m_pPlayer->GetClient() || !m_hstrSurfaceSprite) return;

	DFLOAT fXPan, fYPan, fXScale, fYScale;
	m_pPlayer->GetClient()->GetPolyGridTextureInfo(m_hObject, &fXPan, &fYPan, &fXScale, &fYScale);

	DFLOAT fDeltaTime = m_pPlayer->GetClient()->GetFrameTime();

	// Scale in X direction...

	if (m_fXScaleDuration > 0.0f)
	{
		if (m_bScalingXUp)
		{
			fXScale += fDeltaTime * (m_fXScaleMax - m_fXScaleMin) / m_fXScaleDuration;
			if (fXScale > m_fXScaleMax) 
			{
				fXScale = m_fXScaleMax;
				m_bScalingXUp = DFALSE;
			}
		}
		else
		{
			fXScale -= fDeltaTime * (m_fXScaleMax - m_fXScaleMin) / m_fXScaleDuration;
			if (fXScale < m_fXScaleMin) 
			{
				fXScale = m_fXScaleMin;
				m_bScalingXUp = DTRUE;
			}
		}
	}

	// Scale in Y direction...

	if (m_fYScaleDuration > 0.0f)
	{
		if (m_bScalingYUp)
		{
			fYScale += fDeltaTime * (m_fYScaleMax - m_fYScaleMin) / m_fYScaleDuration;
			if (fYScale > m_fYScaleMax) 
			{
				fYScale = m_fYScaleMax;
				m_bScalingYUp = DFALSE;
			}
		}
		else
		{
			fYScale -= fDeltaTime * (m_fYScaleMax - m_fYScaleMin) / m_fYScaleDuration;
			if (fYScale < m_fYScaleMin) 
			{
				fYScale = m_fYScaleMin;
				m_bScalingYUp = DTRUE;
			}
		}
	}

	m_pPlayer->GetClient()->SetPolyGridTextureInfo(m_hObject, m_fXPan, m_fYPan, fXScale, fYScale);
}


// ----------------
// PrecalculatePlasma
//		Precalculates the plasma motion of the polygrid
// ----------------
void PolyGrid::PrecalculatePlasma()
{
	DDWORD width, height;
	int halfWidth, halfHeight, x, y;
	char *pData;
	DVector *pColorTable;
	float scale, val, maxDistSqr, testDist, t;
	int i;

	if(!g_bSinTableInitted)
	{
		scale = (MATH_CIRCLE / 255.0f) * 3.0f;
		for(i=0; i < 256; i++)
		{
			val = (float)i * scale;
			g_SinTable[i] = (char)(sin(val) * 128.0f);
		}
	
		g_bSinTableInitted = DTRUE;
	}


	// Fill in the distance grid.
	m_pPlayer->GetClient()->GetPolyGridInfo(m_hObject, &pData, &width, &height, &pColorTable);

	m_DistanceGrid = (DBYTE*)malloc(width*height);
	halfWidth = width >> 1;
	halfHeight = height >> 1;
	maxDistSqr = (float)(halfWidth*halfWidth + halfHeight*halfHeight);
	for(y=0; y < (int)height; y++)
	{
		for(x=0; x < (int)width; x++)
		{
			testDist = (float)((x-halfWidth)*(x-halfWidth) + (y-halfHeight)*(y-halfHeight));
			t = testDist / maxDistSqr;
			m_DistanceGrid[y*height+x] = (DBYTE)(t * 255.0f);
		}
	}
}

// ----------------
// SetPolyGridPalette
//		Sets up the Polygrid palette
// ----------------
void PolyGrid::SetPolyGridPalette(ColorRamp *pRamps, int nRamps)
{
	float t;
	DDWORD width, height;
	char *pData;
	DVector *pColorTable, color1, color2;
	int ramp, i, index1, index2;

	// Randomize the poly grid values.
	m_pPlayer->GetClient()->GetPolyGridInfo(m_hObject, &pData, &width, &height, &pColorTable);

	// Make the color table go from white to black.
	for(ramp=0; ramp < nRamps-1; ramp++)
	{
		index1 = pRamps[ramp].m_Index;
		index2 = pRamps[ramp+1].m_Index;
		VEC_COPY(color1, pRamps[ramp].m_Color);
		VEC_COPY(color2, pRamps[ramp+1].m_Color);

		for(i=index1; i < index2; i++)
		{
			t = (float)(i - index1) / (index2 - index1);
			VEC_LERP(pColorTable[i], color1, color2, t);
		}
	}
}

// ----------------
// UpdatePlasma
//		Updates the plasma deformation of the polygrid.
// ----------------
void PolyGrid::UpdatePlasma()
{
	DDWORD width, height, x, y;
	char *pData, *pCur;
	DVector *pColorTable;
	DBYTE count, *pDistanceGrid;

	m_fCount += 50.0f * m_pPlayer->GetClient()->GetFrameTime();
	count = (DBYTE)m_fCount;

	// Randomize the poly grid values.
	m_pPlayer->GetClient()->GetPolyGridInfo(m_hObject, &pData, &width, &height, &pColorTable);
	for(y=0; y < height; y++)
	{
		pCur = pData + (y*width);
		
		pDistanceGrid = &m_DistanceGrid[y*width];
		x = width;
		while(x--)
		{
			*pCur = g_SinTable[(DBYTE)((*pDistanceGrid) + count)];
			++pDistanceGrid;
			++pCur;
		}
	}
}

