
// --------------------------------
// File    : Torch.cpp
// Function: Class for the Torch client-side special effect.
//         : Based on Darren Reid's extensions to the original TorchFX demo.
// --------------------------------

#include "de_client.h"
#include "torch.h"

DBOOL Torch::Init(ClientPlayer *pPlayer, HLOCALOBJ hObj, DBYTE byType, HMESSAGEREAD hMessage)
{
	DVector		pos;
	DVector		Center2;
	float		tRad1, tRad2, randNum;
	int			Speed1, Speed2;
	double		Phase1, Phase2;
	float		LightRadius, LightDuration;
	DBOOL		bComplex;

	m_pPlayer = pPlayer;

	m_pPlayer->GetClient()->GetObjectPos(hObj, &pos);

	// Unpack the message.
	LightRadius = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	LightDuration = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	tRad1 = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	tRad2 = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
	Speed1 = (int)(m_pPlayer->GetClient()->ReadFromMessageDWord(hMessage));
	Speed2 = (int)(m_pPlayer->GetClient()->ReadFromMessageDWord(hMessage));
	bComplex = (m_pPlayer->GetClient()->ReadFromMessageByte(hMessage) != 0);

	// Give the offset position for the center
	// of the second helical path
	Center2.x = 0;
	Center2.z = 0;
	Center2.y = 36;

	randNum = (float)rand() / RAND_MAX;;
	Phase1 = randNum*MATH_CIRCLE;
	randNum = (float)rand() / RAND_MAX;;
	Phase2 = randNum*MATH_CIRCLE;

	StartSystem(m_pPlayer->GetClient(), &pos, LightRadius, LightDuration, Center2, tRad1, tRad2, Speed1, Speed2, Phase1, Phase2, bComplex);

	return DTRUE;
}


void Torch::StartSystem(ClientDE *pClient, DVector *mainpos,
		float lightradius, float lightduration, DVector center2, float Rad1, float Rad2,
		int spiral1, int spiral2,
		double Phase1, double Phase2, DBOOL complex)
{
	ObjectCreateStruct			createStruct;
	float						randNum;
	DVector						color, pos, vel;
	int							nParticles;

	m_pClient = pClient;

	if (!m_bInited) 
	{
		// Create our particle system
		INIT_OBJECTCREATESTRUCT(createStruct);
		createStruct.m_Flags = FLAG_VISIBLE;
		createStruct.m_ObjectType = OT_PARTICLESYSTEM;
		m_hParticles = m_pClient->CreateObject(&createStruct);

		// Set the position
		m_pClient->SetObjectPos(m_hParticles, mainpos);

		m_pClient->SetupParticleSystem(m_hParticles, "textures\\flare1.dtx", 0.0f, 0, 300);

		// Set up our various properties
		m_LightRadius = lightradius;
		m_LightDuration = lightduration;
		m_bSpiral = complex;

		m_center_Pos.x = mainpos->x;
		m_center_Pos.y = mainpos->y;
		m_center_Pos.z = mainpos->z;
		
		VEC_ADD(m_center2_Pos, center2, m_center_Pos);
			
		m_spiral1_Rate = spiral1;
		m_spiral2_Rate = spiral2;
		m_Rad1 = Rad1;
		m_Rad2 = Rad2;
		m_Phase1 = Phase1;
		m_Phase2 = Phase2;
		
		// Build initial system
		nParticles = 20;
		m_bInited = 1;
	
		for(int i=0; i < nParticles; i++)
		{
			randNum = (float)rand() / RAND_MAX;
			color.x = (127.0f * randNum) + 127.0f;
			randNum = (float)rand() / RAND_MAX;
			color.y = (127.0f * randNum) + 127.0f;
			color.z = 0.0f;

			randNum = (float)rand() / RAND_MAX;
			pos.x = (randNum * 10.0f) - 5.0f;
			randNum = (float)rand() / RAND_MAX;
			pos.z = (randNum * 10.0f) - 5.0f;
			pos.y = 0;

			randNum = (float)rand() / RAND_MAX;
			vel.y = (randNum * 10.0f) + 25.0f;
			randNum = (float)rand() / RAND_MAX;
			vel.x = (randNum * 1.5f);
			randNum = (float)rand() / RAND_MAX;
			vel.z = (randNum * 1.5f);

			randNum = (float)rand() / RAND_MAX;
		
			m_pClient->AddParticle(m_hParticles, &pos, &vel, &color, (randNum * 1.0f) + 0.2f);
		}

		// Create our lightsource
		INIT_OBJECTCREATESTRUCT(createStruct);
		createStruct.m_Flags = FLAG_VISIBLE | FLAG_SOLID;
		createStruct.m_ObjectType = OT_LIGHT;
		createStruct.m_Pos = *mainpos;

		m_hLight = m_pClient->CreateObject(&createStruct);
		m_pClient->SetLightColor(m_hLight,1.0f,1.0f,0.0f);
		m_pClient->SetLightRadius(m_hLight, lightradius);
	}
}

DBOOL Torch::Update()
{
	// Sanity check
	if (m_bInited)
	{
		DEParticle *pHead2,*pTail2, *pTemp2;
		DVector v1, v2, d1, d2, d3;
		float dist, dist2, dist3, percent;
		double theta, cos_theta, sin_theta;
		double cos_theta2, sin_theta2, cos_theta3, sin_theta3;

		// Increment our cycle counter
		m_fCycle = m_fCycle + 1.0f; 

		// Only if we're a complicated torch do we do this math.
		if (m_bSpiral)
		{
			
			// Give our particles new positions
			theta = m_Phase1+((int)m_fCycle%m_spiral1_Rate)*MATH_CIRCLE/m_spiral1_Rate;
			cos_theta = cos(theta);
			sin_theta = sin(theta);
			
			cos_theta2 = cos_theta*m_Rad1;
			sin_theta2 = sin_theta*m_Rad1;
			
			theta = m_Phase2+((int)m_fCycle%m_spiral2_Rate)*MATH_CIRCLE/m_spiral2_Rate;
			cos_theta = cos(theta);
			sin_theta = sin(theta);
			cos_theta3 = cos_theta*m_Rad2;
			sin_theta3 = sin_theta*m_Rad2;
			// spin the attractor points about the mainpos
			// Translate to the origin
			VEC_SUB(d1, m_center2_Pos, m_center_Pos);
			// Spin
			d1.x = (float)cos_theta2;
			d1.z = (float)sin_theta2;

			d2.x = (float)cos_theta3;
			d2.z = (float)sin_theta3;
			d2.y = (float)(d1.y/2.5);
			// Translate back
			VEC_ADD(m_center2_Pos, d1, m_center_Pos);
			VEC_ADD(d3, d2, m_center_Pos);

			m_pClient->GetParticles(m_hParticles, &pHead2, &pTail2);
			
			pTemp2 = pHead2;
			while(pTemp2 != pTail2)
			{
				// Get particle position.
				m_pClient->GetParticlePos(m_hParticles, pTemp2, &v1);

				// m_Pos is just an offset from the particle object,
				// so we want our real position
				VEC_ADD(v1, v1, m_center_Pos);

				// Get our particle velocity
				VEC_COPY(v2, pTemp2->m_Velocity);

				// Calculate the distance and direction from the C2 attractor
				dist = VEC_DIST(v1, m_center2_Pos);
				VEC_SUB(d1, m_center2_Pos, v1);
				
				dist3 = VEC_DIST(v1, d3);
				VEC_SUB(d3, d3, v1);
				
				if(d3.y > 0)
				{	
					// We are still below the first attractor
					// Calculate the falloff factor (cosine)
					if(dist3 > 25) percent = 0;
					else if(dist3 < 3) percent = 1.0;
					else
					{
						dist3 = (float)((dist3 - 1)/(25 - 3));
						percent = (float)((cos(dist3*MATH_PI) + 1.0)*0.5);
					}
					VEC_NORM(d3);
					VEC_MULSCALAR(d2, d3, percent*30.1f);
					d2.y /= 10;

					VEC_ADD(pTemp2->m_Velocity, d2, v2);
				}
				else
				{
					// We are above the first attractor, so we
					// use the second attractor.
					// Calculate the falloff factor (cosine)
				
					if(dist > 35) percent = 0;
					else if(dist < 3) percent = 1.0;
					else
					{
						dist2 = (float)((dist - 1)/(35 - 3));
						percent = (float)((cos(dist2*MATH_PI) + 1.0)*0.5);
					}
					VEC_NORM(d1);
					VEC_MULSCALAR(d2, d1, (percent)*5.1f);
					if(d2.y < 0) d2.y = 0.0f;
					else d2.y /= 25.0f;

					VEC_ADD(pTemp2->m_Velocity, d2, v2);
				}
				pTemp2 = pTemp2->m_pNext;
			}
		}

		// Whew!  On with the rest of the torch.

		// If we've gone through one light duration, flicker the light and add more particles
		if (m_fCycle > m_LightDuration) 
		{
			DVector						color, pos, vel;
			int							nParticles;
			float randNum, randNum2, temp;

			temp = m_LightRadius / 3;

			// Create our flicker color
			randNum = (float)rand() / RAND_MAX;
			randNum = (0.5f * randNum) + 0.5f;
			randNum2 = (float)rand() / RAND_MAX;
			randNum2 = (0.5f * randNum2) + 0.5f;
			m_pClient->SetLightColor(m_hLight,randNum,randNum2,0.0f);			

			// And flicker radius
			randNum = (float)rand() / RAND_MAX;
			randNum = (temp * randNum) + (temp * 2);
			m_pClient->SetLightRadius(m_hLight,randNum);						

			nParticles = 8;

			for (int i = 0; i < nParticles; i++)
			{
				if(i%2)
				{
					randNum = (float)rand() / RAND_MAX;
					color.x = (127.0f * randNum) + 127.0f;
					randNum = (float)rand() / RAND_MAX;
					color.y = (127.0f * randNum) + 127.0f;
					color.z = 0.0f;

					randNum = (float)rand() / RAND_MAX;
					pos.x = (randNum * 10.0f) - 5.0f;
					randNum = (float)rand() / RAND_MAX;
					pos.z = (randNum * 10.0f) - 5.0f;
					pos.y = 4.0f;

					randNum = (float)rand() / RAND_MAX;
					vel.y = (randNum * 10.0f) + 25.0f;
					randNum = (float)rand() / RAND_MAX;
					vel.x = (randNum * 1.5f);
					randNum = (float)rand() / RAND_MAX;
					vel.z = (randNum * 1.5f);

					randNum = (float)rand() / RAND_MAX;
		
					m_pClient->AddParticle(m_hParticles, &pos, &vel, &color, (randNum * 0.9f) + 0.2f);
				}
				else if(!(i%8))
				{
					randNum = (float)rand() / RAND_MAX;
					color.x = (10.0f * randNum);
					randNum = (float)rand() / RAND_MAX;
					color.y = (10.0f * randNum);
					randNum = (float)rand() / RAND_MAX;
					color.z = (150.0f * randNum) + 104.0f;
	
					randNum = (float)rand() / RAND_MAX;
					pos.x = (randNum * 6.0f) - 3.0f;
					randNum = (float)rand() / RAND_MAX;
					pos.z = (randNum * 6.0f) - 3.0f;
					pos.y = 0;

					randNum = (float)rand() / RAND_MAX;
					vel.y = (randNum * 10.0f) + 20.0f;
					randNum = (float)rand() / RAND_MAX;
					vel.x = (randNum * 1.5f);
					randNum = (float)rand() / RAND_MAX;
					vel.z = (randNum * 1.5f);

					randNum = (float)rand() / RAND_MAX;
		
					m_pClient->AddParticle(m_hParticles, &pos, &vel, &color, (randNum * 0.2f) + 0.2f);
				}
				else
				{
					randNum = (float)rand() / RAND_MAX;
					color.x = (100.0f * randNum) + 154.0f;
					randNum = (float)rand() / RAND_MAX;
					color.y = (50.0f * randNum) + 50.0f;
					color.z = 0.0f;
	
					randNum = (float)rand() / RAND_MAX;
					pos.x = (randNum * 8.0f) - 4.0f;
					randNum = (float)rand() / RAND_MAX;
					pos.z = (randNum * 8.0f) - 4.0f;
					pos.y = 1.0f;

					randNum = (float)rand() / RAND_MAX;
					vel.y = (randNum * 10.0f) + 15.0f;
					randNum = (float)rand() / RAND_MAX;
					vel.x = (randNum * 1.5f);
					randNum = (float)rand() / RAND_MAX;
					vel.z = (randNum * 1.5f);

					randNum = (float)rand() / RAND_MAX;
		
					m_pClient->AddParticle(m_hParticles, &pos, &vel, &color, (randNum * 0.6f) + 0.2f);
				}
			}
		}
	}
	return DTRUE;
}

void Torch :: DestroySystem ( )
{
	if (m_bInited) {
		m_pClient->DeleteObject(m_hParticles);
		m_pClient->DeleteObject(m_hLight);
	}
}
