
// --------------------------------
// File    : WaterSpray.cpp
// Function: WaterSpray effect class.
// --------------------------------


#include "de_client.h"
#include "waterspray.h"

WaterSpray::WaterSpray() : Effect()
{
	// Do nothing
}

// ----------------
// Init
//		Initializes the WaterSpray.
// ----------------
DBOOL WaterSpray::Init(ClientPlayer *pPlayer, HLOCALOBJ hObj, DBYTE byType, HMESSAGEREAD hMessage)
{
	ObjectCreateStruct createStruct;
	float randNum;
	DVector pos, vel, color;

	m_pPlayer = pPlayer;
	m_hServerObject = hObj;

	// Read our parameters from the HMESSAGE
	m_pPlayer->GetClient()->ReadFromMessageVector(hMessage,&m_vDims);
	m_byThick = m_pPlayer->GetClient()->ReadFromMessageByte(hMessage);

	// Create our object.
	INIT_OBJECTCREATESTRUCT(createStruct);
	createStruct.m_Flags = FLAG_VISIBLE;
	createStruct.m_ObjectType = OT_PARTICLESYSTEM;

	m_pPlayer->GetClient()->GetObjectPos(hObj,&createStruct.m_Pos);
	m_pPlayer->GetClient()->GetObjectRotation(hObj,&createStruct.m_Rotation);

	m_hObject = m_pPlayer->GetClient()->CreateObject(&createStruct);

	m_pPlayer->GetClient()->SetupParticleSystem(m_hObject, "textures\\flare3.dtx", 0.2f, 0, 300);

	// Set up the initial particles
	for(int i=0; i < m_byThick; i++)
	{
		// Give us a color within a given range
		randNum = (float)rand() / RAND_MAX;
		color.x = (50.0f * randNum) + 205.0f;
		randNum = (float)rand() / RAND_MAX;
		color.y = (50.0f * randNum) + 205.0f;
		randNum = (float)rand() / RAND_MAX;
		color.z = (50.0f * randNum) + 205.0f;

		// Give us a location within our dims
		randNum = (float)rand() / RAND_MAX;
		pos.x = (randNum * m_vDims.x * 2.0f) - m_vDims.x;
		randNum = (float)rand() / RAND_MAX;
		pos.y = (randNum * m_vDims.y * 2.0f) - m_vDims.y;
		randNum = (float)rand() / RAND_MAX;
		pos.z = (randNum * m_vDims.z * 2.0f) - m_vDims.z;

		// Give us a velocity...
		vel.y = 0.0f;
		randNum = (float)rand() / RAND_MAX;
		vel.x = (randNum * 5.0f);
		randNum = (float)rand() / RAND_MAX;
		vel.z = (randNum * 10.0f) + 2.0f;

		// And make the particles.
		randNum = (float)rand() / RAND_MAX;
		m_pPlayer->GetClient()->AddParticle(m_hObject, &pos, &vel, &color, (randNum * 4.0f) + 0.2f);
	}

	m_iCounter = 0;

	return DTRUE;
}

DBOOL WaterSpray::Update()
{
	// Sanity check!
	if(!m_hObject || !m_pPlayer->GetClient()) return DFALSE;

	// Don't update if not drawn :)
	if (!(m_pPlayer->GetClient()->GetObjectFlags(m_hObject) & FLAG_WASDRAWN)) 
	{
		return DTRUE;
	}

	// Increment counter
	m_iCounter++;

	// If the counter is too high, add more particles, just like before.
	if (m_iCounter > 4)
	{
		float randNum;
		DVector color, pos, vel;

		m_iCounter = 0;

		for(int i=0; i < (m_byThick / 2); i++)
		{
			randNum = (float)rand() / RAND_MAX;
			color.x = (127.0f * randNum) + 127.0f;
			randNum = (float)rand() / RAND_MAX;
			color.y = (127.0f * randNum) + 127.0f;
			randNum = (float)rand() / RAND_MAX;
			color.z = (127.0f * randNum) + 127.0f;

			randNum = (float)rand() / RAND_MAX;
			pos.x = (randNum * m_vDims.x * 2.0f) - m_vDims.x;
			randNum = (float)rand() / RAND_MAX;
			pos.y = (randNum * m_vDims.y * 2.0f) - m_vDims.y;
			randNum = (float)rand() / RAND_MAX;
			pos.z = (randNum * m_vDims.z * 2.0f) - m_vDims.z;

			vel.y = 0.0f;
			randNum = (float)rand() / RAND_MAX;
			vel.x = (randNum * 5.0f) - 2.5f;
			randNum = (float)rand() / RAND_MAX;
			vel.z = (randNum * 10.0f) + 2.0f;

			randNum = (float)rand() / RAND_MAX;

			m_pPlayer->GetClient()->AddParticle(m_hObject, &pos, &vel, &color, (randNum * 4.0f) + 0.2f);
		}
	}

	return DTRUE;
}

// ----------------
// Destroy
//		Nukes the waterspray.
// ----------------
void WaterSpray::Destroy()
{
	if (!m_hObject) return;

	m_pPlayer->GetClient()->DeleteObject(m_hObject);
}