
// --------------------------------
// File    : sample_clientshell.h
// Function: Class definition for actual sample shell
// --------------------------------

#include <stdio.h>
#include "de_client.h"
#include "sample_clientshell.h"
#include "demo_msgtypes.h"

#include "windows.h"
#include "winbase.h"

// One of our various command types, defined in Autoexec.cfg
#define COMMAND_QUIT			17


// ---- Begin portion that should always be present ----

ClientShellDE* CreateClientShell()
{
	return ((ClientShellDE*)new CSampleClientShell);
}


void DeleteClientShell(ClientShellDE *pShell)
{
	delete ((CSampleClientShell*)pShell);
}

SETUP_CLIENTSHELL();

// ---- End portion that should always be present ----


// --------------------------------------------------------------- //
// CSampleClientShell functions.
// --------------------------------------------------------------- //

CSampleClientShell::CSampleClientShell()
{
	m_hCamera = DNULL;
}

// ----------------
// OnEngineInitialized
//		Called when Engine is first initialized.
// ----------------
void CSampleClientShell::OnEngineInitialized()
{
	StartGameRequest request;
	HCONSOLEVAR hVar;

	// Clear out our request structure.
	memset(&request,0,sizeof(request));

	// Give our player a pointer to the client...
	m_Player.Init(GetClientDE());

	// ...and give our Effects Manager a pointer to player.
	m_EffectMgr.Init(&m_Player);

	// Look for the +runworld command-line parameter and start a world based on that.
	hVar = GetClientDE()->GetConsoleVar("runworld");
	request.m_pWorldName = GetClientDE()->GetVarValueString(hVar);
	request.m_Type = STARTGAME_NORMAL;

	// Actually send the game start request.
	GetClientDE()->StartGame(&request);

}

// ----------------
// SetupSound
//		Called by the engine with a InitSoundInfo pointer.
//		Change the data in the InitSoundInfo structure to alter
//		the way sound is set up.
// ----------------
void CSampleClientShell::SetupSound(InitSoundInfo *pSoundInfo)
{
	// We're not using sound in this demo, set the voices to 0.
	pSoundInfo->m_nNumStaticVoices = 0;
	pSoundInfo->m_nNumDynamicVoices = 0;
}


// ----------------
// OnEnterWorld
//		Called when the client enters a game world.
// ----------------
void CSampleClientShell::OnEnterWorld()
{
	ObjectCreateStruct createStruct;

	// Create our camera object.
	INIT_OBJECTCREATESTRUCT(createStruct);
	createStruct.m_ObjectType = OT_CAMERA;
	m_hCamera = GetClientDE()->CreateObject(&createStruct);

	// Give our player a handle to the camera.
	m_Player.SetCamera(m_hCamera);

	m_bFirst = DTRUE;
}


// ----------------
// OnExitWorld
//		Called when the client exits a world.
// ----------------
void CSampleClientShell::OnExitWorld()
{
	// Eat our camera
	if(m_hCamera)
	{
		GetClientDE()->DeleteObject(m_hCamera);
		m_Player.SetCamera(DNULL);
	}

	// Eat all our effects.
	m_EffectMgr.DestroyAll();
}

void CSampleClientShell::PreUpdate()
{
	if (m_bFirst)
	{
		m_Player.ResetCamera();
		m_hCamera = DNULL;
		m_bFirst = DFALSE;
	}
}

// ----------------
// Update
//		Main client update loop.
// ----------------
void CSampleClientShell::Update()
{
	// Update our player
	m_Player.Update();

	// Update any effects
	m_EffectMgr.Update();
}


// ----------------
// OnCommandOn
//		Called when a command is triggered.  Most of our
//		commands are handled in the ClientPlayer::Update() function,
//		so this one is pretty empty.  In anything other than a basic
//		demo, you'd probably have more here, though.
// ----------------
void CSampleClientShell::OnCommandOn(int command)
{
	if(command == COMMAND_QUIT)
	{
		GetClientDE()->Shutdown();
	}

	m_Player.OnCommandOn(command);
}

// ----------------
// SpecialEffectNotify
//		Called when the client receives a special effects message from the server.
// ----------------
void CSampleClientShell::SpecialEffectNotify(HLOCALOBJ hObj, HMESSAGEREAD hMessage)
{
	// Pass this message into our Effect Manager and let it deal with it.
	m_EffectMgr.Handler(hObj, hMessage);
}





