#include "de_server.h"
#include "player.h"

#include <stdio.h>


// Command IDs (mapped to keys in autoexec.cfg).
#define COMMAND_FORWARD			0
#define COMMAND_BACKWARD		1
#define COMMAND_LEFT			10
#define COMMAND_RIGHT			11
#define COMMAND_JUMP			20
#define COMMAND_STRAFE_LEFT		22
#define COMMAND_STRAFE_RIGHT	23
#define COMMAND_CROUCH			24
#define COMMAND_STRAFE			9
#define COMMAND_SPEED			7



DDWORD Player::EngineMessageFn(DDWORD messageID, void *pData, DFLOAT lData)
{
	float moveSpeed = 120.0f;
	float terminalVelocity = 400.0f;
	DVector velocity, forward, right, up, temp;
	DRotation rotation;
	ObjectCreateStruct *pObjectCreateStruct;
	DVector dims, dims_current, dims2;
	StandingInfo sInfo = {NULL, NULL};
	float strafeSpeed = 85.0f;

	// Give us some dimensions.
	VEC_SET(dims, 7, 25, 7);
	//Set up some crouch dimensions
	VEC_SET(dims2, 7, 10, 7);


	// Create a switch/case statement to handle the various message types
	// we can receive from the engine.  Check serverobj_de.h around line 80 or so
	// for a list of the message types and their definitions.
	switch(messageID)
	{
		// Handle any setup we need before the object is actually created.  Our
		// pData can be cast from a void * to an ObjectCreateStruct * and then
		// populated with any of the specific data we want in it.
		case MID_PRECREATE:
		{
			// Cast our pData to an ObjectCreateStruct pointer
			pObjectCreateStruct = (ObjectCreateStruct*)pData;
			
			// Set up the properties we want for the object.
			pObjectCreateStruct->m_ObjectType = OT_CAMERA;
			pObjectCreateStruct->m_Flags |= FLAG_SOLID | FLAG_GRAVITY | FLAG_STAIRSTEP | FLAG_SHADOW;
			VEC_SET(pObjectCreateStruct->m_Pos,5.0f,80.0f,5.0f);
			sprintf(pObjectCreateStruct->m_Filename,"Models\\ophelia.abc");
			sprintf(pObjectCreateStruct->m_SkinName,"Textures\\ophelia.dtx");
			break;
		}

		// Handle things on the first update cycle.
		case MID_INITIALUPDATE:
		{
			// Tell Lithtech to update us as soon as possible.
			GetServerDE()->SetNextUpdate(m_hObject, 0.001f);
			
			// Give us some dimensions so we can actually bump into things and whatnot.
			GetServerDE()->SetObjectDims(m_hObject, &dims);

			// Give us some mass so physics work!
			GetServerDE()->SetObjectMass(m_hObject, 200.0f);

			break;
		}

		// Handle things during a normal engine update cycle.
		case MID_UPDATE:
		{
			// Check what commands are on and apply movement as appropriate
			GetServerDE()->GetVelocity(m_hObject, &velocity);
			GetServerDE()->GetObjectRotation(m_hObject, &rotation);
			GetServerDE()->GetRotationVectors(&rotation, &up, &right, &forward);

			// Get the object's current dimensions.
			GetServerDE()->GetObjectDims(m_hObject, &dims_current);
/*
			if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_CROUCH))
			{
				if(dims_current.y > dims2.y)
				{
					dims_current.y -= 1.0f;
					GetServerDE()->SetObjectDims(m_hObject, &dims_current);
					
					temp.x = up.x;
					temp.z = up.z;
					temp.y = up.y + 0.1f;
					VEC_ADD(velocity, velocity, temp);
				}
			}
			else 
			{
				if(dims_current.y < dims.y)
				{
					dims_current.y += 1.0f;
					GetServerDE()->SetObjectDims(m_hObject, &dims_current);
					
					temp.x = up.x;
					temp.z = up.z;
					temp.y = up.y + 10.0f;
					VEC_ADD(velocity, velocity, temp);
				}
			}
			*/
			
			if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_FORWARD))
			{
				//VEC_MULSCALAR(temp, forward, moveSpeed);

				//Check for strafing. If we are, modify our movement speed
				if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_STRAFE_LEFT)\
					||GetServerDE()->IsCommandOn(m_hClient, COMMAND_STRAFE_RIGHT))
				{
					//we ARE strafing...modify our speed
					temp.x = forward.x;
					temp.z = forward.z;
					temp.y = 0;
					VEC_NORM(temp);
					temp.x *= strafeSpeed; 
					temp.z *= strafeSpeed;

					//temp.x = forward.x * strafeSpeed;
					//temp.z = forward.z * strafeSpeed;
					temp.y = forward.y;
				}
				else
				{
					temp.x = forward.x;
					temp.z = forward.z;
					temp.y = 0;
					VEC_NORM(temp);
					temp.x *= moveSpeed;
					temp.z *= moveSpeed;

					//temp.x = forward.x * moveSpeed;
					//temp.z = forward.z * moveSpeed;
					temp.y = forward.y;					
				}

				VEC_ADD(velocity, velocity, temp);
			}
/*
			if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_JUMP))
			{
				//first, check to see if we are standing
				//on anything
				GetServerDE()->GetStandingOn(m_hObject, &sInfo);
				if(sInfo.m_hObject)
				{

					//if we ARE on something, jump!
					//VEC_MULSCALAR(temp, up, 600.0f);
					
					temp.x = up.x;
					temp.z = up.z;
					temp.y = up.y + 600;
					

					VEC_ADD(velocity, velocity, temp);
				}
			}
			*/

			if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_BACKWARD))
			{
				//VEC_MULSCALAR(temp, forward, -moveSpeed);
				
				//Check for strafing. If we are, modify our movement speed
				if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_STRAFE_LEFT)\
					||GetServerDE()->IsCommandOn(m_hClient, COMMAND_STRAFE_RIGHT))
				{
					//we ARE strafing...modify our speed
					//VEC_MULSCALAR(temp, forward, -strafeSpeed);
					temp.x = forward.x;
					temp.z = forward.z;
					temp.y = 0;
					VEC_NORM(temp);
					temp.x *= (-strafeSpeed/2); 
					temp.z *= (-strafeSpeed/2);

					//temp.x = forward.x * (-strafeSpeed/2);
					//temp.z = forward.z * (-strafeSpeed/2);
					temp.y = forward.y;
					
				}
				else
				{
					//VEC_MULSCALAR(temp, forward, -moveSpeed);
					temp.x = forward.x;
					temp.z = forward.z;
					temp.y = 0;
					VEC_NORM(temp);
					temp.x *= (-moveSpeed/2); 
					temp.z *= (-moveSpeed/2);

					//temp.x = forward.x * (-moveSpeed/2);
					//temp.z = forward.z * (-moveSpeed/2);
					temp.y = forward.y;
					
				}

				VEC_ADD(velocity, velocity, temp);
			}

			if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_STRAFE_LEFT))
			{
				
				//VEC_MULSCALAR(temp, right, -moveSpeed);

				//Check for Moving. If we are, modify our strafe speed
				if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_FORWARD)\
					||GetServerDE()->IsCommandOn(m_hClient, COMMAND_BACKWARD))
				{
					//we ARE strafing...modify our speed
					//VEC_MULSCALAR(temp, right, -strafeSpeed);
					
					temp.x = right.x * -strafeSpeed;
					temp.z = right.z * -strafeSpeed;
					temp.y = right.y;
					
				}
				else
				{
					//VEC_MULSCALAR(temp, right, -moveSpeed);
					
					temp.x = right.x * -moveSpeed;
					temp.z = right.z * -moveSpeed;
					temp.y = right.y;
					
				}

				VEC_ADD(velocity, velocity, temp);
			}

			if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_STRAFE_RIGHT))
			{	
				//VEC_MULSCALAR(temp, right, moveSpeed);

				//Check for Moving. If we are, modify our strafe speed
				if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_FORWARD)\
					||GetServerDE()->IsCommandOn(m_hClient, COMMAND_BACKWARD))
				{
					//we ARE strafing...modify our speed
					//VEC_MULSCALAR(temp, right, -strafeSpeed);
				
					temp.x = right.x * strafeSpeed;
					temp.z = right.z * strafeSpeed;
					temp.y = right.y;
					
				}
				else
				{
					//VEC_MULSCALAR(temp, right, moveSpeed);
					
					temp.x = right.x * moveSpeed;
					temp.z = right.z * moveSpeed;
					temp.y = right.y;
				
				}

				VEC_ADD(velocity, velocity, temp);
				
			}

			// Slow down for when they aren't pressing keys.
			VEC_MULSCALAR(velocity, velocity, 0.8f);
			if(VEC_MAG(velocity) < 0.05f)
				VEC_SET(velocity, 0, 0, 0);

			// Update the object's velocity.
			GetServerDE()->SetVelocity(m_hObject, &velocity);
			
			// Register for another update..
			GetServerDE()->SetNextUpdate(m_hObject, 0.001f);
			break;
		}
	}
	
	// Pass the message on down.
	return BaseClass::EngineMessageFn(messageID, pData, lData);
}


// Expose the Player class for Lithtech.
BEGIN_CLASS(Player)
// Properties would go here if this were an object we wanted level
// designers to be able to edit them from within the editor.  We don't
// want them putting players in the level, so we don't expose any
// properties, and we set the CF_HIDDEN flag to make it not usable
// in the editor.
END_CLASS_DEFAULT_FLAGS(Player, BaseClass, NULL, NULL, CF_HIDDEN)


