// --------------------------------
// File    : TorchSource.cpp
// Function: Dummy object used as the location/dimensions for the
//         : torch client-side effect.
// --------------------------------

#include <stdio.h>

#include "de_server.h"
#include "torchsource.h"
#include "demo_msgtypes.h"

// Our simple lens flare source object

DDWORD TorchSource::EngineMessageFn(DDWORD messageID, void *pData, DFLOAT lData)
{
	HMESSAGEWRITE hWrite;

	switch(messageID)
	{
		// This is where the variables from the level editor are read.
		case MID_PRECREATE:
		{
			GenericProp gProp;

			if (GetServerDE()->GetPropGeneric("LightRadius",&gProp) == DE_OK)
				LightRadius = gProp.m_Float;
			else
				LightRadius = 200.0f;

			if (GetServerDE()->GetPropGeneric("LightDuration",&gProp) == DE_OK)
				LightDuration = gProp.m_Float;
			else
				LightDuration = 3.0f;

			if (GetServerDE()->GetPropGeneric("Rad1",&gProp) == DE_OK)
				Rad1 = gProp.m_Float;
			else
				Rad1 = 3.27f;

			if (GetServerDE()->GetPropGeneric("Rad2",&gProp) == DE_OK)
				Rad2 = gProp.m_Float;
			else
				Rad2 = 6.13f;

			if (GetServerDE()->GetPropGeneric("spiral1_Rate",&gProp) == DE_OK)
				spiral1_Rate = gProp.m_Long;
			else
				spiral1_Rate = 23;

			if (GetServerDE()->GetPropGeneric("spiral2_Rate",&gProp) == DE_OK)
				spiral2_Rate = gProp.m_Long;
			else
				spiral2_Rate = 33;

			if (GetServerDE()->GetPropGeneric("Complex",&gProp) == DE_OK)
				Complex = gProp.m_Bool;
			else
				Complex = DFALSE;

			break;
		}

		// Our initial update.. 
		// Setup the special effect message.
		case MID_INITIALUPDATE:
		{
			// Send all our relevant data to the client.
			hWrite = GetServerDE()->StartSpecialEffectMessage(this);
			GetServerDE()->WriteToMessageByte(hWrite, DEMO_MSG_TORCH_ADD);
			GetServerDE()->WriteToMessageFloat(hWrite, LightRadius);
			GetServerDE()->WriteToMessageFloat(hWrite, LightDuration);
			GetServerDE()->WriteToMessageFloat(hWrite, Rad1);
			GetServerDE()->WriteToMessageFloat(hWrite, Rad2);
			GetServerDE()->WriteToMessageDWord(hWrite, spiral1_Rate);
			GetServerDE()->WriteToMessageDWord(hWrite, spiral1_Rate);
			GetServerDE()->WriteToMessageByte(hWrite, (Complex ? 1 : 0));
			GetServerDE()->EndMessage(hWrite);
			break;
		}
	}

	// Pass the message on down to our parent.
	return BaseClass::EngineMessageFn(messageID, pData, lData);
}

// Class definition.
BEGIN_CLASS(TorchSource)
	ADD_REALPROP(LightRadius,200.0f)
	ADD_REALPROP(LightDuration,3.0f)
	ADD_REALPROP(Rad1,3.27f)
	ADD_REALPROP(Rad2,6.13f)
	ADD_LONGINTPROP(spiral1_Rate,23)
	ADD_LONGINTPROP(spiral2_Rate,33)
	ADD_BOOLPROP(Complex,DFALSE)
END_CLASS_DEFAULT(TorchSource, BaseClass, NULL, NULL)
