
#include <stdio.h>
#include "de_client.h"
#include "sample_clientshell.h"


// --------------------------------------------------------------- //
// Command IDs (mapped to keys in autoexec.cfg).
// --------------------------------------------------------------- //

#define COMMAND_LEFT		10
#define COMMAND_RIGHT		11
#define COMMAND_LOOKUP		15
#define COMMAND_LOOKDOWN	16
#define COMMAND_QUIT		17



// --------------------------------------------------------------- //
// These functions create the client shell for DirectEngine.
// --------------------------------------------------------------- //

ClientShellDE* CreateClientShell()
{
	return ((ClientShellDE*)new CSampleClientShell);
}


void DeleteClientShell(ClientShellDE *pShell)
{
	delete ((CSampleClientShell*)pShell);
}

SETUP_CLIENTSHELL();



// --------------------------------------------------------------- //
// CSampleClientShell functions.
// --------------------------------------------------------------- //

CSampleClientShell::CSampleClientShell()
{
	m_hCamera = DNULL;
}

void CSampleClientShell::OnEngineInitialized()
{
	StartGameRequest request;
	HCONSOLEVAR hVar;

	// Zero-out our request for safety.
	memset(&request,0,sizeof(request));

	// Look for the +runworld command-line parameter and start a world based on that.
	hVar = GetClientDE()->GetConsoleVar("runworld");
	request.m_pWorldName = GetClientDE()->GetVarValueString(hVar);
	request.m_Type = STARTGAME_NORMAL;
	GetClientDE()->StartGame(&request);
}

void CSampleClientShell::SetupSound(InitSoundInfo *pSoundInfo)
{
	// We're not using sound in this demo, set the voices to 0.
	pSoundInfo->m_nNumStaticVoices = 0;
	pSoundInfo->m_nNumDynamicVoices = 0;
}

void CSampleClientShell::OnEnterWorld()
{
	ObjectCreateStruct createStruct;


	INIT_OBJECTCREATESTRUCT(createStruct);
	createStruct.m_ObjectType = OT_CAMERA;
	m_hCamera = GetClientDE()->CreateObject(&createStruct);

	m_Yaw = m_Pitch = m_Roll = 0.0f;
}


void CSampleClientShell::OnExitWorld()
{
	if(m_hCamera)
	{
		GetClientDE()->DeleteObject(m_hCamera);
		m_hCamera = DNULL;
	}
}


void CSampleClientShell::Update()
{
	HLOCALOBJ hClientObj;
	DVector pos;
	DRotation rotation;
	float radiansPerSec = 2.5f;
	DFLOAT offsets[3];
	HMESSAGEWRITE hMessage;

	// If we're in the world, move and rotate our camera to 
	// line up with the (server-side) player object.
	if(m_hCamera)
	{
		hClientObj = GetClientDE()->GetClientObject();
		if(hClientObj)
		{
			GetClientDE()->GetObjectPos(hClientObj, &pos);
			GetClientDE()->SetObjectPos(m_hCamera, &pos);
			
			// Update our orientation.  We do this on the client and send the
			// angles to the server so mouse orientation is zero latency.
			if(GetClientDE()->IsCommandOn(COMMAND_LEFT))
			{
				m_Pitch -= radiansPerSec * GetClientDE()->GetFrameTime();
			}

			if(GetClientDE()->IsCommandOn(COMMAND_RIGHT))
			{
				m_Pitch += radiansPerSec * GetClientDE()->GetFrameTime();
			}

			if(GetClientDE()->IsCommandOn(COMMAND_LOOKUP))
			{
				m_Yaw -= radiansPerSec * GetClientDE()->GetFrameTime();
			}

			if(GetClientDE()->IsCommandOn(COMMAND_LOOKDOWN))
			{
				m_Yaw += radiansPerSec * GetClientDE()->GetFrameTime();
			}

			GetClientDE()->GetAxisOffsets(offsets);

			m_Pitch += offsets[0] / 100.0f;
			m_Yaw += offsets[1] / 100.0f;

			// Update our camera.
			GetClientDE()->SetupEuler(&rotation, m_Yaw, m_Pitch, m_Roll);
			GetClientDE()->SetObjectRotation(m_hCamera, &rotation);

			// Send the orientation to the server.  The server will update the
			// player object rotation.
			hMessage = GetClientDE()->StartMessage(1);
			GetClientDE()->WriteToMessageRotation(hMessage, &rotation);
			GetClientDE()->EndMessage(hMessage);
		}
	}
}


void CSampleClientShell::OnCommandOn(int command)
{
	if(command == COMMAND_QUIT)
	{
		GetClientDE()->Shutdown();
	}
}









