// ----------------------------------------------------------------------- //
//
// MODULE  : CPP_CLIENTSHELL_DE.H
//
// PURPOSE : C++ DE client shell class definition
//
// CREATED : 9/17/97
//
// ----------------------------------------------------------------------- //

#ifndef __CPP_CLIENTSHELL_DE_H__
#define __CPP_CLIENTSHELL_DE_H__


#include "clientshell_de.h"
#include "cpp_client_de.h"



/////////////////////////////////////////////////////////////////////
// C++ ClientShell interface. This class contains the virtual methods 
// that DirectEngine will call to notify you of things.
/////////////////////////////////////////////////////////////////////

// See ClientShellDE.h for function docs.
class CClientShellDE
{
	public :

		CClientShellDE();
		virtual ~CClientShellDE() {}

		CClientDE* GetClientDE() const;


	protected :

		virtual void OnCommandOn(int command) {}
		virtual void OnCommandOff(int command) {}

		virtual void OnKeyDown(int key, int rep) {}
		virtual void OnKeyUp(int key) {}

		virtual void OnMessage(DBYTE messageID, HMESSAGEREAD hMessage) {}

		virtual void OnModelKey(HLOCALOBJ hObj, ArgList *pArgs) {}

		virtual void PreLoadWorld(char *pWorldName) {}
		virtual void OnEnterWorld() {}
		virtual void OnExitWorld() {}

		virtual void SpecialEffectNotify(HLOCALOBJ hObj, HMESSAGEREAD hMessage) {}

		virtual void OnObjectRemove(HLOCALOBJ hObj) {}

		virtual void PreUpdate() {}
		virtual void Update() {}
		virtual void PostUpdate() {}

		virtual void OnEngineInitialized() {}
		virtual void OnEngineTerm() {}

		virtual void SetupSound( InitSoundInfo *pSoundInitInfo ) {}

	protected :  // Data members

		// VERY Important that this is the first data member.  Pointers to this 
		// class and pointers to ClientShellDE::m_clientShell MUST BE the same. 		
		ClientShellDE m_clientShell;
	

	private :

		// The following static functions are called by the server, and should
		// NOT be called directly.
		static void	_OnCommandOn(LPCLIENTSHELLDE pShell, int command);
		static void	_OnCommandOff(LPCLIENTSHELLDE pShell, int command);
		static void	_OnKeyDown(LPCLIENTSHELLDE pShell, int key, int rep);
		static void	_OnKeyUp(LPCLIENTSHELLDE pShell, int key);
		static void	_OnMessage(LPCLIENTSHELLDE pShell, DBYTE messageID, HMESSAGEREAD hMessage);
		static void _OnModelKey(LPCLIENTSHELLDE pShell, HLOCALOBJ hObj, ArgList *pArgs);
		static void _PreLoadWorld(LPCLIENTSHELLDE pShell, char *pWorldName);
		static void	_OnEnterWorld(LPCLIENTSHELLDE pShell);
		static void	_OnExitWorld(LPCLIENTSHELLDE pShell);
		static void _SpecialEffectNotify(LPCLIENTSHELLDE pShell, HLOCALOBJ hObj, HMESSAGEREAD hMessage);
		static void _OnObjectRemove(LPCLIENTSHELLDE pShell, HLOCALOBJ hObj);
		static void	_PreUpdate(LPCLIENTSHELLDE pShell);
		static void	_Update(LPCLIENTSHELLDE pShell);
		static void	_PostUpdate(LPCLIENTSHELLDE pShell);
		static void _OnEngineInitialized(LPCLIENTSHELLDE pShell);
		static void _OnEngineTerm(LPCLIENTSHELLDE pShell);
		static void _SetupSound(LPCLIENTSHELLDE pShell, InitSoundInfo *pSoundInitInfo );
}; 
	
// Inlines...

INLINE_FN CClientShellDE::CClientShellDE()
{
	// Set up ClientShellDE function pointers...
	m_clientShell.SpecialEffectNotify = _SpecialEffectNotify;
	m_clientShell.OnObjectRemove = _OnObjectRemove;
	m_clientShell.OnCommandOn	= _OnCommandOn;
	m_clientShell.OnCommandOff	= _OnCommandOff;
	m_clientShell.OnKeyDown		= _OnKeyDown;
	m_clientShell.OnKeyUp		= _OnKeyUp;
	m_clientShell.OnMessage		= _OnMessage;
	m_clientShell.PreLoadWorld	= _PreLoadWorld;
	m_clientShell.OnEnterWorld	= _OnEnterWorld;
	m_clientShell.OnModelKey	= _OnModelKey;
	m_clientShell.OnExitWorld	= _OnExitWorld;
	m_clientShell.PreUpdate		= _PreUpdate;
	m_clientShell.Update		= _Update;
	m_clientShell.PostUpdate	= _PostUpdate;
	m_clientShell.OnEngineInitialized = _OnEngineInitialized;
	m_clientShell.OnEngineTerm = _OnEngineTerm;
	m_clientShell.SetupSound = _SetupSound;
}

INLINE_FN CClientDE* CClientShellDE::GetClientDE() const
{ 
	return ((CClientDE*)m_clientShell.m_pClientDE); 
}

INLINE_FN void CClientShellDE::_OnCommandOn(LPCLIENTSHELLDE pShell, int command)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->OnCommandOn(command);
}

INLINE_FN void CClientShellDE::_OnCommandOff(LPCLIENTSHELLDE pShell, int command)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->OnCommandOff(command);
}

INLINE_FN void CClientShellDE::_OnKeyDown(LPCLIENTSHELLDE pShell, int key, int rep)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->OnKeyDown(key, rep);
}

INLINE_FN void CClientShellDE::_OnKeyUp(LPCLIENTSHELLDE pShell, int key)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->OnKeyUp(key);
}

INLINE_FN void CClientShellDE::_OnMessage(LPCLIENTSHELLDE pShell, DBYTE messageID, HMESSAGEREAD hMessage)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->OnMessage(messageID, hMessage);
}

INLINE_FN void CClientShellDE::_OnModelKey(LPCLIENTSHELLDE pShell, HLOCALOBJ hObj, ArgList *pArgList)
{
	((CClientShellDE*)pShell)->OnModelKey(hObj, pArgList);
}

INLINE_FN void CClientShellDE::_PreLoadWorld(LPCLIENTSHELLDE pShell, char *pWorldName)
{
	((CClientShellDE*)pShell)->PreLoadWorld(pWorldName);
}

INLINE_FN void CClientShellDE::_OnEnterWorld(LPCLIENTSHELLDE pShell)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->OnEnterWorld();
}

INLINE_FN void CClientShellDE::_OnExitWorld(LPCLIENTSHELLDE pShell)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->OnExitWorld();
}

INLINE_FN void CClientShellDE::_SpecialEffectNotify(LPCLIENTSHELLDE pShell, HLOCALOBJ hObj, HMESSAGEREAD hMessage)
{
	((CClientShellDE*)pShell)->SpecialEffectNotify(hObj, hMessage);
}

INLINE_FN void CClientShellDE::_OnObjectRemove(LPCLIENTSHELLDE pShell, HLOCALOBJ hObj)
{
	((CClientShellDE*)pShell)->OnObjectRemove(hObj);
}

INLINE_FN void CClientShellDE::_PreUpdate(LPCLIENTSHELLDE pShell)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->PreUpdate();
}

INLINE_FN void CClientShellDE::_Update(LPCLIENTSHELLDE pShell)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->Update();
}

INLINE_FN void CClientShellDE::_PostUpdate(LPCLIENTSHELLDE pShell)
{
	CClientShellDE *pCShell = (CClientShellDE*)pShell;
	pCShell->PostUpdate();
}

INLINE_FN void CClientShellDE::_OnEngineInitialized(LPCLIENTSHELLDE pShell)
{
	((CClientShellDE*)pShell)->OnEngineInitialized();
}

INLINE_FN void CClientShellDE::_OnEngineTerm(LPCLIENTSHELLDE pShell)
{
	((CClientShellDE*)pShell)->OnEngineTerm();
}

INLINE_FN void CClientShellDE::_SetupSound(LPCLIENTSHELLDE pShell, InitSoundInfo *pSoundInitInfo )
{
	if(pShell)
	{
		((CClientShellDE*)pShell)->SetupSound( pSoundInitInfo );
	}
}


#endif  // __CPP_CLIENTSHELL_DE_H__

