// ----------------------------------------------------------------------- //
//
// MODULE  : CPP_SERVERSHELL_DE.H
//
// PURPOSE : C++ DE server shell class definition
//
// CREATED : 9/17/97
//
// ----------------------------------------------------------------------- //

#ifndef __CPP_SERVERSHELL_DE_H__
#define __CPP_SERVERSHELL_DE_H__

#include "servershell_de.h"

// Forward declarations...

class BaseClass;
class CServerDE;

/////////////////////////////////////////////////////////////////////
// C++ ServerShellDE interface.  Derive your server object from this.
/////////////////////////////////////////////////////////////////////

class CServerShellDE
{
	public :

		CServerShellDE();
		virtual ~CServerShellDE() {}

		CServerDE* GetServerDE() const;

	protected :  // These methods should be over-written

		// Notification when new clients come in.
		// You must create an object to represent the client.
		// It uses the object's position to determine what the client can see.
		virtual void	OnAddClient(HCLIENT hClient) {}
		virtual void	OnRemoveClient(HCLIENT hClient) {}

		virtual LPBASECLASS	OnClientEnterWorld(HCLIENT hClient) {return NULL;}
		virtual void		OnClientExitWorld(HCLIENT hClient) {}

		// Called before and after you switch worlds.
		virtual void	PreStartWorld(DBOOL bSwitchingWorlds) {}
		virtual void	PostStartWorld() {}		

		// Incoming message notification.
		virtual void OnMessage(HCLIENT hSender, DBYTE messageID, HMESSAGEREAD hMessage) {}
		virtual void OnObjectMessage(LPBASECLASS pSender, DDWORD messageID, HMESSAGEREAD hMessage) {}		

		// Command notification.
		virtual void	OnCommandOn(HCLIENT hClient, int command) {}
		virtual void	OnCommandOff(HCLIENT hClient, int command) {}

		// Update loop callback.. do whatever you like in here.
		// Time since the last Update() call is passed in.
		virtual void	Update(DFLOAT timeElapsed) {}


	private :  // Data members

		// VERY Important that this first data member.  Pointers to this class and
		// pointers to CServerShellDE::m_serverShell MUST BE the same. 
		ServerShellDE	m_serverShell;  


	private :

		// The following static functions are called by the server, and should
		// NOT be called directly.
		static void _OnAddClient(struct ServerShellDE_t *pShell, HCLIENT hClient);
		static void _OnRemoveClient(struct ServerShellDE_t *pShell, HCLIENT hClient);

		static LPBASECLASS _OnClientEnterWorld(struct ServerShellDE_t *pShell, HCLIENT hClient);
		static void _OnClientExitWorld(struct ServerShellDE_t *pShell, HCLIENT hClient);

		static void _PreStartWorld(struct ServerShellDE_t *pShell, DBOOL bSwitchingWorlds);
		static void _PostStartWorld(struct ServerShellDE_t *pShell);
		static void _OnMessage(struct ServerShellDE_t *pShell, HCLIENT hSender, DBYTE messageID, HMESSAGEREAD hMessage);
		static void	_OnObjectMessage(struct ServerShellDE_t *pShell, LPBASECLASS pSender, DDWORD messageID, HMESSAGEREAD hMessage);
		static void _OnCommandOn(struct ServerShellDE_t *pShell, HCLIENT hClient, int command);
		static void _OnCommandOff(struct ServerShellDE_t *pShell, HCLIENT hClient, int command);
		static void _Update(struct ServerShellDE_t *pShell, DFLOAT timeElapsed);
};


// Inlines...

inline CServerShellDE::CServerShellDE()
{
	// Set up ServerShellDE function pointers...
	m_serverShell.OnAddClient		= _OnAddClient;
	m_serverShell.OnRemoveClient	= _OnRemoveClient;
	m_serverShell.OnClientEnterWorld = _OnClientEnterWorld;
	m_serverShell.OnClientExitWorld = _OnClientExitWorld;
	m_serverShell.PreStartWorld		= _PreStartWorld;
	m_serverShell.PostStartWorld	= _PostStartWorld;
	m_serverShell.OnMessage			= _OnMessage;
	m_serverShell.OnObjectMessage	= _OnObjectMessage;
	m_serverShell.OnCommandOn		= _OnCommandOn;
	m_serverShell.OnCommandOff		= _OnCommandOff;
	m_serverShell.Update			= _Update;
}

inline CServerDE* CServerShellDE::GetServerDE() const
{ 
	return ((CServerDE*)m_serverShell.m_pServerDE); 
}


inline void CServerShellDE::_OnAddClient(struct ServerShellDE_t *pShell, HCLIENT hClient)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->OnAddClient(hClient);
	}
}

inline void CServerShellDE::_OnRemoveClient(struct ServerShellDE_t *pShell, HCLIENT hClient)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->OnRemoveClient(hClient);
	}
}

inline LPBASECLASS CServerShellDE::_OnClientEnterWorld(struct ServerShellDE_t *pShell, HCLIENT hClient)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		return (LPBASECLASS)pCSShell->OnClientEnterWorld(hClient);
	}

	return NULL;
}

inline void CServerShellDE::_OnClientExitWorld(struct ServerShellDE_t *pShell, HCLIENT hClient)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->OnClientExitWorld(hClient);
	}
}

inline void CServerShellDE::_PreStartWorld(struct ServerShellDE_t *pShell, DBOOL bSwitchingWorlds)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->PreStartWorld(bSwitchingWorlds);
	}
}

inline void CServerShellDE::_PostStartWorld(struct ServerShellDE_t *pShell)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->PostStartWorld();
	}
}

inline void CServerShellDE::_OnMessage(struct ServerShellDE_t *pShell, HCLIENT hSender, DBYTE messageID, HMESSAGEREAD hMessage)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->OnMessage(hSender, messageID, hMessage);
	}
}

inline void CServerShellDE::_OnObjectMessage(struct ServerShellDE_t *pShell, LPBASECLASS pSender, DDWORD messageID, HMESSAGEREAD hMessage)
{
	if(pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->OnObjectMessage(pSender, messageID, hMessage);
	}
}

inline void CServerShellDE::_OnCommandOn(struct ServerShellDE_t *pShell, HCLIENT hClient, int command)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->OnCommandOn(hClient, command);
	}
}

inline void CServerShellDE::_OnCommandOff(struct ServerShellDE_t *pShell, HCLIENT hClient, int command)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->OnCommandOff(hClient, command);
	}
}

inline void CServerShellDE::_Update(struct ServerShellDE_t *pShell, DFLOAT timeElapsed)
{
	if (pShell)
	{
		CServerShellDE *pCSShell = (CServerShellDE*)pShell;
		pCSShell->Update(timeElapsed);
	}
}


#endif  // __CPP_SERVERSHELL_DE_H__
